import javax.vecmath.*;

public class Condition
{
	String parameter;
	String operation;
	ParticleGroup[] particles;
	float[] values;
	Boundary boundary;

	public Condition(String param, String op, ParticleGroup[] parts, float[] vals, Boundary b)
	{
		parameter = param;
		operation = op;
		particles = parts;
		values = vals;
		boundary = b;
	} 
	
	public boolean check()
	{
		// Check the distance between two particles
		if (parameter.equals("dist"))
		{
			if (particles.length<2 || values.length<1)
			{ return false; }
			
			Particle[] outerParts = particles[1].getParticles();
			
			float bestDist = -1f;
			Particle bestPart = null;
			for (int i=0; i<outerParts.length; i++)
			{
				// Calculate distance
				float[] vect = boundary.vectorBetween 		
								(particles[0].getCurrentParticle().getPosition(), 
								 outerParts[i].getPosition());
				float dist = new Vector3f(vect[0], vect[1], vect[2]).length();
				
				if (operation.equals("greaterthan") && (dist > bestDist || bestDist < 0f))
				{
					bestDist = dist;
					bestPart = outerParts[i];
				}
				
				if (operation.equals("lessthan") && (dist < bestDist || bestDist < 0f))
				{
					bestDist = dist;
					bestPart = outerParts[i];
				}
			}
			
			if (bestDist < 0f)
			{ return false; }
			
			if (operation.equals("greaterthan") && bestDist > values[0])
			{ return true; }
			if (operation.equals("lessthan") && bestDist < values[0])
			{ 
				particles[1].setCurrentParticle(bestPart);
				return true; 
			}
		}
	
		return false;
	}
}