import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;

import CoordinateLayout.*;

public class ControlPanel extends JPanel
{
	SimulationGenerator simGen;
	StartPauseButton startPause;
	IterationTextField iterations;
	TimeStepUnitMenu timeUnitMenu;
	JLabel frameLab, simElapsed; 
	JTabbedPane tabbedPane;
	
	public ControlPanel(SimulationGenerator sg, JTabbedPane t)
	{
		super();
		simGen = sg;
		tabbedPane = t;
		
		setBorder(new EtchedBorder());
		setLayout(new CoordinateLayout(new Dimension(32,14),4,4));
		
		startPause = new StartPauseButton(simGen, this);
		add(startPause, new Coordinate(1,1,16,3));
		add(new ResetButton(simGen, this), new Coordinate(16,1,31,3));
		
		simElapsed = new JLabel("0.00", JLabel.RIGHT);
		frameLab = new JLabel("-");
		(simGen.getSimulationGroup()).setElapsedLabel(simElapsed);
		(simGen.getSimulationGroup()).setFrameLabel(frameLab);
		
		JLabel simElapseUnit = new JLabel("Nanoseconds");
		JLabel frameUnit = new JLabel("Frames / s");
		JLabel frameHeading = new JLabel("Frame rate :");
		//JLabel simTime = new JLabel("Time :");
		 
		add(new JLabel("Time-step per frame :"), new Coordinate(1,3,31,5));
		add(new TimeStepValueTextField(simGen), new Coordinate(1,5,12,7));
		timeUnitMenu = new TimeStepUnitMenu(simGen, simElapseUnit);
		add(timeUnitMenu, new Coordinate(12,5,31,7));
		
		//add(simTime, new Coordinate(1,7,8,9));
		add(frameHeading, new Coordinate(1,9,15,11));
		add(simElapsed, new Coordinate(1,7,16,9));
		add(frameLab, new Coordinate(15,9,20,11));
		add(simElapseUnit, new Coordinate(16,7,31,9));
		add(frameUnit, new Coordinate(20,9,31,11));
		
		add(new JLabel("Iterations :"), new Coordinate(1,11,15,13));
		iterations = new IterationTextField(simGen);
		add(iterations, new Coordinate(15,11,31,13));
		
		simGen.setControlPanel(this);
	}
	
	public JTabbedPane getTabbedPane()
	{
		return tabbedPane;
	}
	
	public StartPauseButton getStartButton()
	{
		return startPause;
	}
	
	public void refresh()
	{
		(simGen.getSimulationGroup()).setElapsedLabel(simElapsed);
		(simGen.getSimulationGroup()).setFrameLabel(frameLab);
		iterations.refresh();
		timeUnitMenu.refresh();
	}
}

class IterationTextField extends JTextField implements KeyListener
{
	SimulationGenerator simGen;
	
	public IterationTextField(SimulationGenerator sg)
	{
		super(6);
		setText("500");
		simGen = sg;
		(simGen.getSimulationGroup()).setIterations(500);
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}
	
	public void keyPressed(KeyEvent ke)
	{}
	
	public void keyTyped(KeyEvent ke)
	{}
	
	public void keyReleased(KeyEvent ke)
	{
		try 
		{
			int i = (int) (Integer.parseInt(getText()));
			(simGen.getSimulationGroup()).setIterations(i);
		}
		catch (NumberFormatException e)
		{}
	}
	
	public void refresh()
	{
		try 
		{
			int i = (int) (Integer.parseInt(getText()));
			(simGen.getSimulationGroup()).setIterations(i);
		}
		catch (NumberFormatException e)
		{}
	}
}

class TimeStepValueTextField extends JTextField implements KeyListener
{
	SimulationGenerator simGen;
	
	public TimeStepValueTextField(SimulationGenerator sg)
	{
		super(6);
		setText("1");
		simGen = sg;
		(simGen.getSimulationGroup()).setTimeValue(1f);
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}
	
	public void keyPressed(KeyEvent ke)
	{}
	
	public void keyTyped(KeyEvent ke)
	{}
	
	public void keyReleased(KeyEvent ke)
	{
		try 
		{
			float f = (float) (Float.parseFloat(getText()));
			(simGen.getSimulationGroup()).setTimeValue(f);
		}
		catch (NumberFormatException e)
		{}
	}
}

// Reset button
class ResetButton extends JButton implements ActionListener
{
	SimulationGenerator simGen;
	ControlPanel controlPanel;
	
	public ResetButton(SimulationGenerator sg, ControlPanel cp)
	{
		super("Reset");
		addActionListener(this);
		simGen = sg;
		controlPanel = cp;
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		(simGen.getSimulationGroup()).enabled(false);
		(simGen.getSimulationGroup()).reset(0);
		controlPanel.getStartButton().setText("Start");
		controlPanel.getTabbedPane().setEnabled(true);
	}
}

// Start/Pause Button
class StartPauseButton extends JButton implements ActionListener
{
	SimulationGenerator simGen;
	ControlPanel panel;
	
	public StartPauseButton(SimulationGenerator sg, ControlPanel p)
	{
		super("Start");
		addActionListener(this);
		simGen = sg;
		panel = p;
		(simGen.getSimulationGroup()).enabled(false);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		if ((simGen.getSimulationGroup()).isEnabled() == false)
		{
			setText("Pause");
			panel.getTabbedPane().setSelectedIndex(0);
			panel.getTabbedPane().setEnabled(false);
			(simGen.getSimulationGroup()).enabled(true);
		}
		else
		{
			setText("Start");
			(simGen.getSimulationGroup()).enabled(false);
		}
	}
}

class TimeStepUnitMenu extends JComboBox implements ItemListener
{
	SimulationGenerator simGen;
	JLabel simElapseUnit;
	
	public TimeStepUnitMenu(SimulationGenerator sg, JLabel seu)
	{
		super();
		simGen = sg;
		simElapseUnit = seu;
		
		addItem("Seconds");
		addItem("Milliseconds");
		addItem("Microseconds");
		addItem("Nanoseconds");
		addItem("Picoseconds");
		addItem("Femtoseconds");
		
		setEditable(false);
		
		setSelectedIndex(3);
		(simGen.getSimulationGroup()).setTimeUnit((float) Math.pow(10.0, -9.0)); 
		
		addItemListener(this);
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();
		SimulationGroup simulationGroup = simGen.getSimulationGroup();
		
		if (s.equals("Seconds")) { simulationGroup.setTimeUnit(1f); }
		if (s.equals("Milliseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -3.0)); }
		if (s.equals("Microseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -6.0)); }
		if (s.equals("Nanoseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -9.0)); }
		if (s.equals("Picoseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -12.0)); }
		if (s.equals("Femtoseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -15.0)); }
		
		simElapseUnit.setText(s);
	}
	
	public void refresh()
	{
		String s = (String) getSelectedItem();
		SimulationGroup simulationGroup = simGen.getSimulationGroup();
		
		if (s.equals("Seconds")) { simulationGroup.setTimeUnit(1f); }
		if (s.equals("Milliseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -3.0)); }
		if (s.equals("Microseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -6.0)); }
		if (s.equals("Nanoseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -9.0)); }
		if (s.equals("Picoseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -12.0)); }
		if (s.equals("Femtoseconds")) 
			{ simulationGroup.setTimeUnit((float) Math.pow(10.0, -15.0)); }
	}
	
}