import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import CoordinateLayout.*;

public class ElectrostaticPanel extends JPanel
{
	
	SimulationGenerator simGen;
	EPObjectTemplateList objectTemplates;
	EPObjectStateList objectStates;
	EPAddStateButton addStateButton;
	EPParticleGroupList groupList1, groupList2;
	EPApplyButton applyButton;
	EPCollisionButton collisionButton;
	EPConnectionButton connectionButton;
	ConnectionTextField connectionVal;
	ConnectionUnitMenu connectionUnit;
	
	public ElectrostaticPanel(SimulationGenerator sgen)
	{
		super();
		
		simGen = sgen; 
		
		setLayout(new BorderLayout());
		JPanel panel = new JPanel();
		panel.setPreferredSize(new Dimension(800, 200));
		panel.setLayout(new CoordinateLayout(new Dimension(33,32),4,4));
		
		objectTemplates = new EPObjectTemplateList(simGen, this);
		
		panel.add(new JLabel("Object templates:"), new Coordinate(1,1,8,4));
		panel.add(objectTemplates, new Coordinate(1,4,8,26));
		
		objectStates = new EPObjectStateList(simGen, this);
		panel.add(new JLabel("Object state:"), new Coordinate(9,1,16,4));
		panel.add(objectStates, new Coordinate(9,4,16,26));
		
		addStateButton = new EPAddStateButton(this);
		panel.add(addStateButton, new Coordinate(9,27,16,31));
		
		
		groupList1 = new EPParticleGroupList(simGen, this);
		panel.add(new JLabel("Start Particle:"), new Coordinate(17,1,24,4));
		panel.add(groupList1, new Coordinate(17,4,24,16));
		
		groupList2 = new EPParticleGroupList(simGen, this);
		panel.add(new JLabel("End Particle:"), new Coordinate(17,16,24,20));
		panel.add(groupList2, new Coordinate(17,20,24,31));
		
		panel.add(new JLabel("Create link:"), new Coordinate(25,1,32,4));
		applyButton = new EPApplyButton(this);
		panel.add(applyButton, new Coordinate(25,8,32,12));
		
		collisionButton = new EPCollisionButton(this);
		panel.add(collisionButton, new Coordinate(25,14,32,18));
		
		connectionVal = new ConnectionTextField();
		connectionUnit = new ConnectionUnitMenu(simGen);
		panel.add(connectionVal, new Coordinate(25,20,28,24));
		panel.add(connectionUnit, new Coordinate(28,20,32,24));
		
		connectionButton = new EPConnectionButton(this);
		panel.add(connectionButton, new Coordinate(25,24,32,28));
		
		add(panel, BorderLayout.WEST);
	}
	
	
	public void refresh()
	{
		objectTemplates.refresh();
		objectStates.refresh();
		groupList1.refresh();
		groupList2.refresh();
	}
	
	
	public ObjectTemplate getSelectedObjectTemplate()
	{
		int select = objectTemplates.getSelectedIndex();
		if (select == -1 || simGen.getObjectTemplates().size() <= select) 
		{ return null; }
		
		return ((ObjectTemplate) (simGen.getObjectTemplates().elementAt(select)));
	}
	
	public int getSelectedState()
	{
		return objectStates.getSelectedIndex();
	}
	
	public EPObjectStateList getObjectStateList()
	{
		return objectStates;
	}
	
	public EPObjectTemplateList getObjectTemplates()
	{
		return objectTemplates;
	}
	
	EPParticleGroupList getStartGroupList()
	{
		return groupList1;
	}
	
	EPParticleGroupList getEndGroupList()
	{
		return groupList2;
	}
	
	public SimulationGenerator getSimGen()
	{
		return simGen;
	}
	
	public float getDistance()
	{
		return (connectionVal.getVal()) * (connectionUnit.getUnit());
	}
}

class ConnectionTextField extends JTextField
{
	
	public ConnectionTextField()
	{
		super(6);
		setText("1");
		
		setHorizontalAlignment(JTextField.RIGHT);
	}
	
	public float getVal()
	{
		try 
		{
			return (float) (Float.parseFloat(getText()));
		}
		catch (NumberFormatException e)
		{ return 0f; }
	}
}

class ConnectionUnitMenu extends JComboBox implements ItemListener
{
	SimulationGenerator simulationGenerator;
	JLabel simElapseUnit;
	float connectionUnit;
	
	public ConnectionUnitMenu(SimulationGenerator sg)
	{
		super();
		simulationGenerator = sg;
		
		addItem("Metres");
		addItem("Millimetres");
		addItem("Micrometres");
		addItem("Nanometres");
		addItem("Picometres");
		addItem("Femtometres");
		
		setEditable(false);
		
		setSelectedIndex(3);
		connectionUnit = (float) Math.pow(10.0, -9.0); 
		
		addItemListener(this);
	}
	
	public float getUnit()
	{
		return connectionUnit;
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();
		
		if (s.equals("Metres")) { connectionUnit = 1f;  }
		if (s.equals("Millimetres")) 
			{ connectionUnit = (float) Math.pow(10.0, -3.0);  }
		if (s.equals("Micrometres")) 
			{ connectionUnit = (float) Math.pow(10.0, -6.0);  }
		if (s.equals("Nanometres")) 
			{ connectionUnit = (float) Math.pow(10.0, -9.0);  }
		if (s.equals("Picometres")) 
			{ connectionUnit = (float) Math.pow(10.0, -12.0);  }
		if (s.equals("Femtometres")) 
			{ connectionUnit = (float) Math.pow(10.0, -15.0);  }
	}
}

// add group template button
class EPConnectionButton extends JButton implements ActionListener
{
	ElectrostaticPanel panel;
	
	public EPConnectionButton(ElectrostaticPanel p)
	{
		super("Create Connection");
		panel = p;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		ObjectTemplate oT = panel.getSelectedObjectTemplate();
		float distance = panel.getDistance();
		if (oT != null)
		{
			int state = panel.getSelectedState();
			int part1 = panel.getStartGroupList().getSelectedIndex();
			int part2 = panel.getEndGroupList().getSelectedIndex();
			if (state >= 0 && part1 >= 0 && part2 >= 0)
			{
				oT.addConnection(part1, part2, distance, state);
			}
		}
		panel.refresh();
	}
}

// add group template button
class EPCollisionButton extends JButton implements ActionListener
{
	ElectrostaticPanel panel;
	
	public EPCollisionButton(ElectrostaticPanel p)
	{
		super("Create Collision");
		panel = p;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		ObjectTemplate oT = panel.getSelectedObjectTemplate();
		if (oT != null)
		{
			int state = panel.getSelectedState();
			int part1 = panel.getStartGroupList().getSelectedIndex();
			int part2 = panel.getEndGroupList().getSelectedIndex();
			if (state >= 0 && part1 >= 0 && part2 >= 0)
			{
				oT.addCollision(part1, part2, state);
			}
		}
		panel.refresh();
	}
}

// add group template button
class EPApplyButton extends JButton implements ActionListener
{
	ElectrostaticPanel panel;
	
	public EPApplyButton(ElectrostaticPanel p)
	{
		super("Create Electrostatic");
		panel = p;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		ObjectTemplate oT = panel.getSelectedObjectTemplate();
		if (oT != null)
		{
			int state = panel.getSelectedState();
			int part1 = panel.getStartGroupList().getSelectedIndex();
			int part2 = panel.getEndGroupList().getSelectedIndex();
			if (state >= 0 && part1 >= 0 && part2 >= 0)
			{
				oT.addElectrostatic(part1, part2, state);
			}
		}
		panel.refresh();
	}
}

class EPParticleGroupList extends JPanel
{
	SimulationGenerator simGen;
	JList pgList;
	ElectrostaticPanel panel;
	
	public EPParticleGroupList(SimulationGenerator sgen, ElectrostaticPanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		pgList = new JList();
		pgList.setVisibleRowCount(3);
		pgList.setFixedCellWidth(150);
		pgList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		pgList.setEnabled(true);
		pgList.setSelectedIndex(-1);
		
				
		pgList.addListSelectionListener(
			new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e)
				{
				}
			}
		);
		
		JScrollPane scrollPane = new JScrollPane(pgList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}	
	
	public void refresh()
	{		
		ObjectTemplate oT = panel.getSelectedObjectTemplate(); 
		int select;
		
		select = pgList.getSelectedIndex();
		Vector names = new Vector();
		if (oT != null)
		{
			Vector groupVect = oT.getParticleGroups();
			for (int i=0; i<groupVect.size(); i++)
			{
				names.add("Group "+i);
			}
		}
		pgList.setListData(names);
		pgList.setSelectedIndex(select);
	}

	public int getSelectedIndex()
	{
		return pgList.getSelectedIndex();
	}
}

class EPObjectStateList extends JPanel
{
	SimulationGenerator simGen;
	JList pgList;
	ElectrostaticPanel panel;
	
	public EPObjectStateList(SimulationGenerator sgen, ElectrostaticPanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		pgList = new JList();
		pgList.setVisibleRowCount(6);
		pgList.setFixedCellWidth(150);
		pgList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		pgList.setEnabled(true);
		
				
		pgList.addListSelectionListener(
			new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e)
				{
				}
			}
		);
		
		JScrollPane scrollPane = new JScrollPane(pgList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}	
	
	public void refresh()
	{		
		ObjectTemplate oT = panel.getSelectedObjectTemplate(); 
		int select;
		
		select = pgList.getSelectedIndex();
		Vector names = new Vector();
		if (oT != null)
		{
			Vector stateVect = oT.getStates();
			for (int i=0; i<stateVect.size(); i++)
			{
				names.add("State "+(i+1));
			}
		}
		pgList.setListData(names);
		pgList.setSelectedIndex(select);
	}

	public int getSelectedIndex()
	{
		return pgList.getSelectedIndex();
	}
}

class EPObjectTemplateList extends JPanel
{
	SimulationGenerator simGen;
	JList objectList;
	ElectrostaticPanel panel;
	
	public EPObjectTemplateList(SimulationGenerator sgen, ElectrostaticPanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		objectList = new JList();
		objectList.setVisibleRowCount(6);
		objectList.setFixedCellWidth(150);
		objectList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		objectList.setEnabled(true);
		
				
		objectList.addListSelectionListener(
			new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e)
				{
					panel.getObjectStateList().refresh();
					panel.getStartGroupList().refresh();
					panel.getEndGroupList().refresh();
				}
			}
		);
		
		JScrollPane scrollPane = new JScrollPane(objectList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}	
	
	public void refresh()
	{		
		Vector objects = simGen.getObjectTemplates();
		
		Vector names = new Vector();
		for (int i=0; i<objects.size(); i++)
		{
			names.add(((ObjectTemplate) (objects.elementAt(i))).getName());
		}
		int select = objectList.getSelectedIndex();
		objectList.setListData(names);
		objectList.setSelectedIndex(select);
	}

	public int getSelectedIndex()
	{
		return objectList.getSelectedIndex();
	}
}

// add group template button
class EPAddStateButton extends JButton implements ActionListener
{
	ElectrostaticPanel panel;
	
	public EPAddStateButton(ElectrostaticPanel p)
	{
		super("New state");
		panel = p;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		ObjectTemplate oT = panel.getSelectedObjectTemplate();
		
		if (oT != null)
		{
			oT.addState();
		}
		panel.refresh();
	}
}
