import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import CoordinateLayout.*;

public class MediumPanel extends JPanel
{
	SimulationGenerator simGen;
	
	public MediumPanel(SimulationGenerator sgen)
	{
		super();
		
		simGen = sgen;
		
		setLayout(new CoordinateLayout(new Dimension(8,4),4,4));
		
		add(new JLabel("Temperature :"), new Coordinate(0,0,8,1));
		TemperatureTextField ttf = new TemperatureTextField(simGen);
		add(ttf, new Coordinate(0,1,3,2));
		TemperatureUnitMenu tempUM = new TemperatureUnitMenu(simGen);
		add(tempUM, new Coordinate(3,1,8,2));
	
		add(new JLabel("Viscosity :"), new Coordinate(0,2,8,3));
		ViscosityTextField vtf = new ViscosityTextField(simGen);
		add(vtf, new Coordinate(0,3,3,4));
		ViscosityUnitMenu viscUM = new ViscosityUnitMenu(simGen);
		add(viscUM, new Coordinate(3,3,8,4));
	}
}

class ViscosityTextField extends JTextField implements KeyListener
{
	SimulationGenerator simGen;
	
	public ViscosityTextField(SimulationGenerator sgen)
	{
		super(6);
		setText("1");
		simGen = sgen;
		simGen.setViscosityValue(1f);
		
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}
	
	public void keyPressed(KeyEvent ke)
	{}
	
	public void keyTyped(KeyEvent ke)
	{}
	
	public void keyReleased(KeyEvent ke)
	{
		try 
		{
			float visc = (float) (Float.parseFloat(getText()));
			simGen.setViscosityValue(visc);
		}
		catch (NumberFormatException e)
		{}
	}
}

class ViscosityUnitMenu extends JComboBox implements ItemListener
{
	SimulationGenerator simGen;
	
	public ViscosityUnitMenu(SimulationGenerator sg)
	{
		super();
		simGen = sg;
		
		addItem("Pa.S");
		addItem("mPa.S");
		addItem("Pa.S");
		
		setEditable(false);
		
		setSelectedIndex(1);
		simGen.setViscosityUnit((float) Math.pow(10.0, -3.0)); 
		
		addItemListener(this);
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();
		
		if (s.equals("Pa.S")) { simGen.setViscosityUnit(1f); }
		if (s.equals("mPa.S")) 
			{ simGen.setViscosityUnit((float) Math.pow(10.0, -3.0)); }
		if (s.equals("Pa.S")) 
			{ simGen.setViscosityUnit((float) Math.pow(10.0, -6.0)); }
	}
}

class TemperatureTextField extends JTextField implements KeyListener
{
	SimulationGenerator simGen;
	
	public TemperatureTextField(SimulationGenerator sgen)
	{
		super(6);
		setText("20");
		simGen = sgen;
		simGen.setTemperature(20f);
		
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}
	
	public void keyPressed(KeyEvent ke)
	{}
	
	public void keyTyped(KeyEvent ke)
	{}
	
	public void keyReleased(KeyEvent ke)
	{
		try 
		{
			float temp = (float) (Float.parseFloat(getText()));
			simGen.setTemperature(temp);
		}
		catch (NumberFormatException e)
		{}
	}
}

class TemperatureUnitMenu extends JComboBox implements ItemListener
{
	SimulationGenerator simGen;
	int temperatureUnit;
	final int CELCIUS = 0;
	final int KELVIN = 1;
	
	public TemperatureUnitMenu(SimulationGenerator sg)
	{
		super();
		simGen = sg;
		
		addItem("Celcius");
		addItem("Kelvin");
		
		setEditable(false);
		
		setSelectedIndex(0);
		temperatureUnit = CELCIUS;
		
		addItemListener(this);
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();
		
		if (s.equals("Celcius"))
		{
			simGen.setTemperatureUnit(CELCIUS);
		}
		else
		{
			simGen.setTemperatureUnit(KELVIN);
		}
	}
}