import java.util.*;

public class ObjectTemplate
{
	Vector particles;
	Vector particlePositions;
	Vector particleGroupOffsets;
	Vector structures;
	Vector particleGroups;
	String name;
	Vector rootParticles;
	Vector rootObjects;

	public ObjectTemplate(Vector obTemplates)
	{
		particles = new Vector();
		particlePositions = new Vector();
		particleGroupOffsets = new Vector();
		structures = new Vector();
		structures.addElement(new StructureTemplate());
		particleGroups = new Vector();
		rootParticles = new Vector();
		rootObjects = new Vector();
		name = "";
		obTemplates.addElement(this);
	}

	public ObjectTemplate()
	{
		particles = new Vector();
		particlePositions = new Vector();
		particleGroupOffsets = new Vector(); 
		structures = new Vector();
		structures.addElement(new StructureTemplate());
		particleGroups = new Vector();
		rootParticles = new Vector();
		rootObjects = new Vector();
		name = "";
	}
	
	public void setName(String s)
	{
		name = s;
	}
	
	public String getName()
	{
		return name;
	}
	
	public int addState()
	{
		return ((StructureTemplate)(structures.elementAt(0))).addState();
	}
	
	public Vector getStates()
	{
		return ((StructureTemplate)(structures.elementAt(0))).getStates();
	}
	
	public void addConnection(int a, int b, float d, int state)
	{
		((StructureTemplate)(structures.elementAt(0))).addConnection(a,b,d,state);
	}	
		
	public void addCollision(int a, int b, int state)
	{
		((StructureTemplate)(structures.elementAt(0))).addCollision(a,b,state);
	}	
	
	public void addHinge(int a, int b, int c, float d, int state)
	{
		((StructureTemplate)(structures.elementAt(0))).addHinge(a,b,c,d,state);
	}
	
	public void addElectrostatic(int a, int b, int state)
	{
		((StructureTemplate)(structures.elementAt(0))).addElectrostatic(a,b,state);
	}
	
	public void addStateChange(int newState, int state)
	{
		((StructureTemplate)(structures.elementAt(0))).addStateChange(newState, state);
	}
	
	public void addCondition(ConditionTemplate c, int stateChange, int state)
	{
		((StructureTemplate)(structures.elementAt(0))).addCondition(c, stateChange, state);
	}
	
	public void addParticle(ParticleTemplate pT, float[] pos)
	{
		particles.addElement(pT);
		particlePositions.addElement(pos);
		rootParticles.addElement(pT);
		
		ParticleGroupTemplate pgt = new ParticleGroupTemplate(particles.size()-1, new Vector());
		particleGroupOffsets.addElement(new Integer(particleGroups.size()));
		addParticleGroup(pgt);
	}
	
	public void addParticleGroup(ParticleGroupTemplate p)
	{
		particleGroups.addElement(p);
	}
	
	public Vector getParticleGroups()
	{
		return particleGroups;
	}	
	
	public void addObject(ObjectTemplate oT, float[] objectPos)
	{
		int incr = particles.size();
		rootObjects.addElement(oT);
		
		Vector parts = oT.getParticles();
		Vector partPos = oT.getParticlePositions();
		for (int i=0; i<parts.size(); i++)
		{
			particles.addElement(parts.elementAt(i));
			float[] particlePos = (float[]) partPos.elementAt(i);
			float[] newPos = new float[3];
			newPos[0] = objectPos[0] + particlePos[0];
			newPos[1] = objectPos[1] + particlePos[1];
			newPos[2] = objectPos[2] + particlePos[2];
			particlePositions.addElement(newPos);
		}
		
		int partGroupIncr = particleGroups.size();
		Vector partGroups = oT.getParticleGroups();
		for (int i=0; i<partGroups.size(); i++)
		{
			ParticleGroupTemplate pg = (ParticleGroupTemplate) (partGroups.elementAt(i));
			particleGroups.addElement(pg.copy(incr));
			if (pg.getCurrentParticle() != -1)
			{
				particleGroupOffsets.addElement(new Integer(incr+i));
			}
		}
		
		Vector structs = oT.getStructures();
		for (int i=0; i<structs.size(); i++)
		{
			StructureTemplate t = (StructureTemplate) (structs.elementAt(i));
			StructureTemplate tCopy = t.copy(incr);
			structures.addElement(tCopy);
		}
	}
	
	public SimObject[] createInstance(float objectPos[])
	{
		Vector particleInstances = new Vector();
		Vector particleGroupInstances = new Vector();
		
		for (int i=0; i<particles.size(); i++)
		{
			ParticleTemplate pT = (ParticleTemplate) particles.elementAt(i);
			float[] particlePos = (float[]) particlePositions.elementAt(i);
			float[] newPos = new float[3];
			newPos[0] = objectPos[0] + particlePos[0];
			newPos[1] = objectPos[1] + particlePos[1];
			newPos[2] = objectPos[2] + particlePos[2];
			particleInstances.addElement(pT.createInstance(newPos));
		}
		
		for (int i=0; i<particleGroups.size(); i++)
		{
			ParticleGroupTemplate pgt = ((ParticleGroupTemplate) (particleGroups.elementAt(i)));
			particleGroupInstances.addElement(
							pgt.createInstance(particleInstances, particleGroupOffsets));
		}
		
		SimObject[] objects = new SimObject[structures.size()];
		for (int i=0; i<structures.size(); i++)
		{
			objects[i] = ((StructureTemplate)(structures.elementAt(i))).
							createInstance(particleGroupInstances, this);
		}
		 
		return objects;
	}
	
	public Vector getStructures()
	{
		return structures;
	}
	
	public StructureTemplate getStructure()
	{
		return (StructureTemplate) (structures.elementAt(0));
	}
	
	public Vector getParticles()
	{
		return particles;
	}
	
	public Vector getRootParticles()
	{
		return rootParticles;
	}
	
	public Vector getRootObjects()
	{
		return rootObjects;
	}
	
	public Vector getParticlePositions()
	{
		return particlePositions;
	}
}