import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import CoordinateLayout.*;

public class ObjectTemplatePanel extends JPanel
{
	
	SimulationGenerator simGen;
	OTParticleList particles;
	OTParticleTemplateList templates;
	OTObjectTemplateList objectTemplates; 
	ComponentPositionTextField[] optf;
	ComponentPositionUnitMenu[] opum;
	ParticleInstanceButton pib;
	OTNewObjectField newObjectName;
	OTAddObjectButton addObjectButton;
	SubObjectInstanceButton soib;
	OTSubObjectTemplateList subObjTemplates;
	OTSubObjectList subObjectList;
	
	public ObjectTemplatePanel(SimulationGenerator sgen)
	{
		super();
		
		simGen = sgen; 
		
		setLayout(new BorderLayout());
		JPanel panel = new JPanel();
		panel.setPreferredSize(new Dimension(800, 200));
		panel.setLayout(new CoordinateLayout(new Dimension(33,32),4,4));
		
	 	pib = new ParticleInstanceButton(simGen, this);
		templates = new OTParticleTemplateList(simGen, this);
		
		soib = new SubObjectInstanceButton(simGen, this);
		subObjTemplates = new OTSubObjectTemplateList(simGen, this);
		
		objectTemplates = new OTObjectTemplateList(simGen, this);
		particles = new OTParticleList(simGen, this);
		subObjectList = new OTSubObjectList(simGen, this);
		
		newObjectName = new OTNewObjectField();
		addObjectButton = new OTAddObjectButton(this);
		
		panel.add(new JLabel("Object templates:"), new Coordinate(1,1,8,4));
		panel.add(objectTemplates, new Coordinate(1,4,8,26));
		panel.add(newObjectName, new Coordinate(1,27,5,31));
		panel.add(addObjectButton, new Coordinate(5,27,8,31));
		
		panel.add(new JLabel("Particle templates:"), new Coordinate(9,1,16,4));
		panel.add(templates, new Coordinate(9,4,16,16));
		panel.add(new JLabel("Sub-object templates:"), new Coordinate(9,16,16,20));
		panel.add(subObjTemplates, new Coordinate(9,20,16,31));
		
		optf = new ComponentPositionTextField[3];
		opum = new ComponentPositionUnitMenu[3]; 
		optf[0] = new ComponentPositionTextField(); 
		optf[1] = new ComponentPositionTextField(); 
		optf[2] = new ComponentPositionTextField(); 
		opum[0] = new ComponentPositionUnitMenu();
		opum[1] = new ComponentPositionUnitMenu();
		opum[2] = new ComponentPositionUnitMenu();
		
		
		panel.add(pib, new Coordinate(17,4,24,8));
		
		panel.add(new JLabel("X :"), new Coordinate(17,10,18,14));
		panel.add(optf[0], new Coordinate(18,10,20,14));
		panel.add(opum[0], new Coordinate(20,10,24,14));
		
		panel.add(new JLabel("Y :"), new Coordinate(17,15,18,19));
		panel.add(optf[1], new Coordinate(18,15,20,19));
		panel.add(opum[1], new Coordinate(20,15,24,19));
		
		panel.add(new JLabel("Z :"), new Coordinate(17,20,18,24));
		panel.add(optf[2], new Coordinate(18,20,20,24));
		panel.add(opum[2], new Coordinate(20,20,24,24));
		
		panel.add(soib, new Coordinate(17,27,24,31));
		
		panel.add(new JLabel("Particles:"), new Coordinate(25,1,32,4));
		panel.add(particles, new Coordinate(25,4,32,16));
		panel.add(new JLabel("Sub-objects:"), new Coordinate(25,16,33,20));
		panel.add(subObjectList, new Coordinate(25,20,32,31));
		
		add(panel, BorderLayout.WEST);
	}
	
	public void refresh()
	{
		particles.refresh();
		templates.refresh();
		subObjTemplates.refresh();
		objectTemplates.refresh();
		subObjectList.refresh();
	}
	
	public ParticleInstanceButton getInstanceButton()
	{
		return pib;
	}
	
	public OTParticleList getParticleList()
	{
		return particles;
	}
	
	public SubObjectInstanceButton getSOInstanceButton()
	{
		return soib;
	}
	
	public OTSubObjectList getSubObjectList()
	{
		return subObjectList;
	}
	
	public OTObjectTemplateList getObjectTemplateList()
	{
		return objectTemplates;
	}
	
	public int getSelectedTemplate()
	{
		return templates.getSelectedIndex();
	}
	
	public int getSelectedSubObjectTemplate()
	{
		return subObjTemplates.getSelectedIndex();
	}
	
	public ObjectTemplate getSelectedObjectTemplate()
	{
		int select = objectTemplates.getSelectedIndex();
		if (select == -1 || simGen.getObjectTemplates().size() <= select) 
		{ return null; }
		
		return ((ObjectTemplate) (simGen.getObjectTemplates().elementAt(select)));
	}
	
	public float[] getPos()
	{
		float[] pos = new float[3];
		
		pos[0] = optf[0].getVal() * opum[0].getUnit();
		pos[1] = optf[1].getVal() * opum[1].getUnit();
		pos[2] = optf[2].getVal() * opum[2].getUnit();
		
		return pos;
	}
	
	public OTNewObjectField getNewObjectName()
	{
		return newObjectName;
	}
	
	public SimulationGenerator getSimGen()
	{
		return simGen;
	}
}

// add object template button
class OTAddObjectButton extends JButton implements ActionListener
{
	ObjectTemplatePanel panel;
	
	public OTAddObjectButton(ObjectTemplatePanel p)
	{
		super("Add");
		panel = p;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		Vector ot = panel.getSimGen().getObjectTemplates();
		
		String newName = panel.getNewObjectName().getVal();
		if (!(newName.equals("")))
		{
			ObjectTemplate newTemplate = new ObjectTemplate(ot);
			newTemplate.setName(newName);
			panel.getObjectTemplateList().refresh();
			panel.getNewObjectName().setVal("");
		}
	}
}

class OTNewObjectField extends JTextField
{
	String val;
	
	public OTNewObjectField()
	{
		super(6);
		setHorizontalAlignment(JTextField.LEFT);
		setText("");
	}
	
	public void setVal(String n)
	{
		setText(n);
	}
	
	public String getVal()
	{
		return getText();
	}
}

class OTObjectTemplateList extends JPanel
{
	SimulationGenerator simGen;
	JList objectList;
	ObjectTemplatePanel panel;
	
	public OTObjectTemplateList(SimulationGenerator sgen, ObjectTemplatePanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		objectList = new JList();
		objectList.setVisibleRowCount(6);
		objectList.setFixedCellWidth(150);
		objectList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		objectList.setEnabled(true);
		
				
		objectList.addListSelectionListener(
			new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e)
				{
					int select = objectList.getSelectedIndex();
					panel.getParticleList().refresh();
					panel.getSubObjectList().refresh();
					if (select == -1)
					{
					}
					else 
					{
					}
				}
			}
		);
		
		JScrollPane scrollPane = new JScrollPane(objectList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}	
	
	public void refresh()
	{		
		Vector objects = simGen.getObjectTemplates();
		
		Vector names = new Vector();
		for (int i=0; i<objects.size(); i++)
		{
			names.add(((ObjectTemplate) (objects.elementAt(i))).getName());
		}
		int select = objectList.getSelectedIndex();
		objectList.setListData(names);
		objectList.setSelectedIndex(select);
	}

	public int getSelectedIndex()
	{
		return objectList.getSelectedIndex();
	}
}

class OTSubObjectTemplateList extends JPanel
{
	SimulationGenerator simGen;
	JList particleList;
	ObjectTemplatePanel panel;
	
	public OTSubObjectTemplateList(SimulationGenerator sgen, ObjectTemplatePanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		particleList = new JList();
		particleList.setVisibleRowCount(3);
		particleList.setFixedCellWidth(140);
		particleList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		particleList.setEnabled(true);
		JScrollPane scrollPane = new JScrollPane(particleList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}	
	
	public void refresh()
	{		
		Vector particles = simGen.getObjectTemplates();
		
		Vector names = new Vector();
		for (int i=0; i<particles.size(); i++)
		{
			names.add(((ObjectTemplate) (particles.elementAt(i))).getName());
		}
		int select = particleList.getSelectedIndex();
		particleList.setListData(names);
		particleList.setSelectedIndex(select);
	}

	public int getSelectedIndex()
	{
		return particleList.getSelectedIndex();
	}
}

class OTSubObjectList extends JPanel
{
	SimulationGenerator simGen;
	JList objectList;
	ObjectTemplatePanel panel;
	
	public OTSubObjectList(SimulationGenerator sgen, ObjectTemplatePanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		objectList = new JList();
		objectList.setVisibleRowCount(3);
		objectList.setFixedCellWidth(140);
		objectList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		objectList.setEnabled(true);
		objectList.setSelectionForeground(new Color(0,0,0));
		objectList.setSelectionBackground(new Color(255,255,255));
		JScrollPane scrollPane = new JScrollPane(objectList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}
	
	public void refresh()
	{	
		ObjectTemplate currTemp = panel.getSelectedObjectTemplate();
		if (currTemp != null)
		{
			Vector objectTemplates = currTemp.getRootObjects();
		
			Vector names = new Vector();
			for (int i=0; i<objectTemplates.size(); i++)
			{
				names.add((i+1)+". "+
					((ObjectTemplate) (objectTemplates.elementAt(i))).getName());
			}
			int select = objectList.getSelectedIndex();
			objectList.setListData(names);
			objectList.setSelectedIndex(select);
		}
	}
}

// sub-object instance button
class SubObjectInstanceButton extends JButton implements ActionListener
{
	ObjectTemplatePanel panel;
	SimulationGenerator simGen;
	
	public SubObjectInstanceButton(SimulationGenerator sg, ObjectTemplatePanel p)
	{
		super("Add sub-object >>>");
		panel = p;
		simGen = sg;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		int selected = panel.getSelectedSubObjectTemplate();
		if (selected != -1)
		{
			ObjectTemplate ot = panel.getSelectedObjectTemplate();
			if (ot != null)
			{
				float[] pos = panel.getPos();
				
				ObjectTemplate pt = (ObjectTemplate) 
										(simGen.getObjectTemplates().elementAt(selected));
		
				ot.addObject(pt, pos);
				panel.getSubObjectList().refresh();
				simGen.updateSimulation();
			}
		}
	}
}

class OTParticleTemplateList extends JPanel
{
	SimulationGenerator simGen;
	JList particleList;
	ObjectTemplatePanel panel;
	
	public OTParticleTemplateList(SimulationGenerator sgen, ObjectTemplatePanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		particleList = new JList();
		particleList.setVisibleRowCount(3);
		particleList.setFixedCellWidth(140);
		particleList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		particleList.setEnabled(true);
		JScrollPane scrollPane = new JScrollPane(particleList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}	
	
	public void refresh()
	{		
		Vector particles = simGen.getParticleTemplates();
		
		Vector names = new Vector();
		for (int i=0; i<particles.size(); i++)
		{
			names.add(((ParticleTemplate) (particles.elementAt(i))).getName());
		}
		int select = particleList.getSelectedIndex();
		particleList.setListData(names);
		particleList.setSelectedIndex(select);
	}

	public int getSelectedIndex()
	{
		return particleList.getSelectedIndex();
	}
}

class OTParticleList extends JPanel
{
	SimulationGenerator simGen;
	JList objectList;
	ObjectTemplatePanel panel;
	
	public OTParticleList(SimulationGenerator sgen, ObjectTemplatePanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		objectList = new JList();
		objectList.setVisibleRowCount(3);
		objectList.setFixedCellWidth(140);
		objectList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		objectList.setEnabled(true);
		objectList.setSelectionForeground(new Color(0,0,0));
		objectList.setSelectionBackground(new Color(255,255,255));
		JScrollPane scrollPane = new JScrollPane(objectList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}
	
	public void refresh()
	{	
		ObjectTemplate currTemp = panel.getSelectedObjectTemplate();
		if (currTemp != null)
		{
			Vector particleTemplates = currTemp.getRootParticles();
		
			Vector names = new Vector();
			for (int i=0; i<particleTemplates.size(); i++)
			{
				names.add((i+1)+". "+
					((ParticleTemplate) (particleTemplates.elementAt(i))).getName());
			}
			int select = objectList.getSelectedIndex();
			objectList.setListData(names);
			objectList.setSelectedIndex(select);
		}
	}
}

// Particle instance button
class ParticleInstanceButton extends JButton implements ActionListener
{
	ObjectTemplatePanel panel;
	SimulationGenerator simGen;
	
	public ParticleInstanceButton(SimulationGenerator sg, ObjectTemplatePanel p)
	{
		super("Add particle >>>");
		panel = p;
		simGen = sg;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		int selected = panel.getSelectedTemplate();
		if (selected != -1)
		{
			ObjectTemplate ot = panel.getSelectedObjectTemplate();
			if (ot != null)
			{
				float[] pos = panel.getPos();
				
				ParticleTemplate pt = (ParticleTemplate) 
										(simGen.getParticleTemplates().elementAt(selected));
		
				ot.addParticle(pt, pos);
				panel.getParticleList().refresh();
				simGen.updateSimulation();
			}
		}
	}
}


class ComponentPositionTextField extends JTextField implements KeyListener
{
	String val;
	
	public ComponentPositionTextField()
	{
		super(6);
		enableField();
		setText("0");
		val = "0";
		
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}
	
	public void disableField()
	{
		setText("");
		val="";
		setEnabled(false);
	}
	
	public void enableField()
	{
		setEnabled(true);
	}
		
	public void keyPressed(KeyEvent ke)
	{}
	
	public void keyTyped(KeyEvent ke)
	{}
	
	public void keyReleased(KeyEvent ke)
	{
		String text = getText();
		try 
		{
			val = "" + (float) (Float.parseFloat(text));
		}
		catch (NumberFormatException e)
		{}
	}
	
	public void setVal(float v)
	{
		val = v + "";
		setText(val);
	}
	
	public float getVal()
	{
		try 
		{
			return (float) (Float.parseFloat(val));
		}
		catch (NumberFormatException e)
		{}
		
		return 0f;
	}
}

class ComponentPositionUnitMenu extends JComboBox implements ItemListener
{
	float radUnit;
	
	public ComponentPositionUnitMenu()
	{
		super();
		
		addItem("Metres");
		addItem("Millimetres");
		addItem("Micrometres");
		addItem("Nanometres");
		addItem("Picometres");
		addItem("Femtometres");
		
		setEditable(false);
		setUnit((float) Math.pow(10.0, -9.0));
	
		addItemListener(this);
	}
	
	public float getUnit()
	{
		return radUnit;
	}
	
	public void setUnit(float unit)
	{
		radUnit = unit;
		
		if (unit < 1.1f && unit > 0.9f) { setSelectedIndex(0); }
		if (unit < (float) Math.pow(10.0, -2.0) && unit > (float) Math.pow(10.0, -4.0)) 
			{ setSelectedIndex(1); }
		if (unit < (float) Math.pow(10.0, -5.0) && unit > (float) Math.pow(10.0, -7.0)) 
			{ setSelectedIndex(2); }
		if (unit < (float) Math.pow(10.0, -8.0) && unit > (float) Math.pow(10.0, -10.0)) 
			{ setSelectedIndex(3); }
		if (unit < (float) Math.pow(10.0, -11.0) && unit > (float) Math.pow(10.0, -13.0)) 
			{ setSelectedIndex(4); }
		if (unit < (float) Math.pow(10.0, -14.0) && unit > (float) Math.pow(10.0, -16.0)) 
			{ setSelectedIndex(5); }
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();
		
		if (s.equals("Metres")) { radUnit = 1f;  }
		if (s.equals("Millimetres")) 
			{ radUnit = (float) Math.pow(10.0, -3.0);  }
		if (s.equals("Micrometres")) 
			{ radUnit = (float) Math.pow(10.0, -6.0);  }
		if (s.equals("Nanometres")) 
			{ radUnit = (float) Math.pow(10.0, -9.0);  }
		if (s.equals("Picometres")) 
			{ radUnit = (float) Math.pow(10.0, -12.0);  }
		if (s.equals("Femtometres")) 
			{ radUnit = (float) Math.pow(10.0, -15.0);  }	
	}
	
	public void enableField()
	{ 
		removeItem("");
		setEnabled(true);
	}
	
	public void disableField()
	{
		addItem("");
		setSelectedItem("");
		setEnabled(false);
	}
}