import java.util.*;
 
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Particle 
{
	// Physical properties
	float[] position = new float[3];
	boolean isStatic;
	Vector states;
	int currentState;
	Vector markedPositions;
	
	// Medium
	Medium medium;
	Boundary boundary;
	
	// Interactions
	Vector connectionList = new Vector();
	Vector collisionList = new Vector();
	Vector interactionList = new Vector();
	Vector hingeList = new Vector();
	
	// Display properties
	TransformGroup j3d;
	//BranchGroup j3dContainer;
	Transform3D trans;
	
	ParticleTemplate template;

	public Particle(float[] pos, ParticleTemplate t)
	{
		position[0] = pos[0];
		position[1] = pos[1];
		position[2] = pos[2];
		
		medium = new Medium();
		boundary = new Boundary("", 1f);
		
		states = new Vector();
		states.addElement("start");
		currentState = 0;
		
		template = t;
		
		markedPositions = new Vector();
		float[] initialPos = {pos[0], pos[1], pos[2]};
		markedPositions.addElement(initialPos);
		
		isStatic = false;
	}
	
	public void setStatic(boolean s)
	{
		isStatic = s;
	}
	
	public boolean isStatic()
	{
		return isStatic;
	}
	
	public void reset(int resetPos)
	{
		float[] resetTo = (float[]) markedPositions.elementAt(resetPos);
		position[0] = resetTo[0];
		position[1] = resetTo[1];
		position[2] = resetTo[2];
	}
	
	public void addState(String name)
	{
		states.addElement(name);
	}
	
	public void setStates(Vector s)
	{
		states = s;
	}
	
	public Vector getStates()
	{
		return states;
	}
	
	public void setCurrentState(int s)
	{
		if (s<states.size())
		{
			currentState = s;
		}
	}
	
	public int getCurrentState()
	{
		return currentState;
	}
	
	public boolean checkTemplateAndState(ParticleTemplate t, int[] s)
	{
		if (!(template == t))
		{ return false; }
		
		for (int i=0; i<s.length; i++)
		{
			if (s[i] == currentState)
			{ return true; }
		}
		
		return false;
	}
	
	public boolean checkParticleState(int[] s)
	{
		for (int i=0; i<s.length; i++)
		{
			if (s[i] == currentState)
			{ return true; }
		}
		return false;
	}
	 
	public void createJ3dObject()
	{
		//j3dContainer = new BranchGroup();
		//j3dContainer.setCapability(BranchGroup.ALLOW_DETACH);
		j3d = new TransformGroup();
		j3d.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	
		/*
		TransparencyAttributes ta = new TransparencyAttributes();
		ta.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);        						
		ta.setTransparencyMode(ta.FASTEST);
		*/

		Appearance a = new Appearance();
		
		// Enable Z-buffering to ensure correct object order
		RenderingAttributes ra = new RenderingAttributes();
 		ra.setDepthBufferEnable(true);
  		a.setRenderingAttributes(ra);
		
        //a.setTransparencyAttributes(ta);
		a.setCapability(Appearance.ALLOW_MATERIAL_READ);
		a.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
		a.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
		a.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
		Material c = new Material();
		c.setCapability(Material.ALLOW_COMPONENT_READ);
		c.setCapability(Material.ALLOW_COMPONENT_WRITE);
		c.setLightingEnable(true);
		
		Color3f colour = template.getColour3f();
		c.setAmbientColor(colour);
		c.setEmissiveColor(new Color3f(0f,0f,0f));
		c.setDiffuseColor(new Color3f(colour.x*0.15f, colour.y*0.15f, colour.z*0.15f));
		c.setSpecularColor(colour);
		c.setShininess(128.0f);
		a.setMaterial(c);
				
		float sphereRadius = boundary.j3dSize(template.getRadius());
	    Sphere j3dSphere = new Sphere(sphereRadius, a);
		j3dSphere.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_READ);
		j3dSphere.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		
		trans = new Transform3D();
		updateScene();
		j3d.addChild(j3dSphere);
	}
	
	public void updateScene()
	{
		boundary.enforce(position);
		
		trans.setTranslation(new Vector3f(boundary.j3dSize(position[0]), 
										  boundary.j3dSize(position[1]),
										  boundary.j3dSize(position[2])));
		j3d.setTransform(trans);
	}
	
	public void addToSimulation(SimulationGroup sg)
	{
		sg.addChild(j3d);
	}
	
	public void updatePosition(Random r)
	{
		float cSize = (float) connectionList.size();
		Particle connectingParticle;
		float[] connectorPosition;
		float connectingDistance;
		float connectorRadius;
		float ratioOfRadii;
		float signedSeparation;
		float threeDSeparation;
		float distanceRatio;
		float movement;
		float[] sep;
		Vector randomParticles;
		
		// Correct particle locations with respect to rigid connections
		randomParticles = getRandomList((int) cSize, r);
		for (int t=0; t<cSize; t++)
		{
			int i = ((Integer) (randomParticles.elementAt(t))).intValue();
			// Get connecting particle details
			connectingParticle = ((Connection) connectionList.elementAt(i)).getParticle();
			connectingDistance = ((Connection) connectionList.elementAt(i)).getDist();
			connectorPosition = connectingParticle.getPosition();
			connectorRadius = connectingParticle.getRadius();
			
			// Adjust the position of this particle by an amount
			// proportional to the distance and radii of the connection particles
			ratioOfRadii = connectorRadius / (connectorRadius + template.getRadius());
			
			//////sep = boundary.vectorBetween(connectorPosition, position);
			
			sep = new float[3];
			sep[0] = connectorPosition[0] - position[0];
			sep[1] = connectorPosition[1] - position[1];
			sep[2] = connectorPosition[2] - position[2];
			threeDSeparation = (new Vector3f(sep[0], sep[1], sep[2])).length();
			
			if (threeDSeparation != 0f)
			{
				for (int j=0; j<3; j++)
				{
					//signedSeparation = boundary.dist(connectorPosition[j], position[j]);
					signedSeparation =  position[j]-connectorPosition[j];
					distanceRatio = ratioOfRadii *
							((threeDSeparation-connectingDistance)/threeDSeparation);
					movement = 0.3f * distanceRatio * signedSeparation;

					position[j] -= movement;
				}
			}
		}
		
		cSize = (float) collisionList.size();
		// Detect and correct inter-particle collisions
		for (int i=0; i<cSize; i++)
		{
			// Get connecting particle details
			connectingParticle = (Particle) collisionList.elementAt(i);
			connectorPosition = connectingParticle.getPosition();
			connectorRadius = connectingParticle.getRadius();
			connectingDistance = template.getRadius() + connectorRadius;
			
			// Adjust the position of this particle by an amount
			// proportional to the distance and radii of the connection particles
			ratioOfRadii = connectorRadius / (connectorRadius + template.getRadius());
			
			//sep = boundary.vectorBetween(connectorPosition, position);
			sep = new float[3];
			sep[0] = connectorPosition[0] - position[0];
			sep[1] = connectorPosition[1] - position[1];
			sep[2] = connectorPosition[2] - position[2];
			threeDSeparation = (new Vector3f(sep[0], sep[1], sep[2])).length();
			
			if (threeDSeparation < connectingDistance)
			{
				for (int j=0; j<3; j++)
				{
				
				    //signedSeparation = boundary.dist(connectorPosition[j], position[j]);
					signedSeparation = position[j] - connectorPosition[j];
					distanceRatio = ratioOfRadii *
							((threeDSeparation-connectingDistance)/threeDSeparation);
					movement = 0.3f * distanceRatio * signedSeparation;

					position[j] -= movement;
				}
			}
		}
	}
	
	public void calculateBrownianDisplacement(float timestep, float[] displacement, Random r)
	{	
		for (int j=0; j<3; j++)
		{
			displacement[j] = medium.calculateBrownianTranslation(template.getRadius(),
																  timestep, r);
		}
	}
	
	public void collisionLimitedMove(float[] vectorCB)
	{
		float cosSquaredAlpha;
		Particle collidingParticle;
		float collidingRadius;
		float[] collidingPosition = new float[3];
		float[] vectorCA = new float[3];
		float magnitudeSquaredCA;
		float magnitudeSquaredCB;
		float perpendicularDistanceSquared;
		float radiiSquared;
		float magnitudeSquaredMaxPerp;
		float magnitudeMax;
		float motionMagnitude;
		float magCAMinusRadii;
		float trimRatio;
		
		magnitudeSquaredCB = (vectorCB[0]*vectorCB[0] + 
						   	  vectorCB[1]*vectorCB[1] + 
							  vectorCB[2]*vectorCB[2]);
							  	
		motionMagnitude = magnitudeSquaredCB;
		
		// If any collisions occur then restrict the movement due to Brownian Motion.
		for (int i=0; i<collisionList.size(); i++)
		{
			collidingParticle = (Particle) collisionList.elementAt(i);
			collidingPosition = collidingParticle.getPosition();
			collidingRadius = collidingParticle.getRadius();
			
			for (int j=0; j<3; j++)
			{
				vectorCA[j] = collidingPosition[j] - position[j];
				//vectorCA[j] = boundary.dist(position[j], collidingPosition[j]);
			}
						
			magnitudeSquaredCA = (vectorCA[0]*vectorCA[0] + 
					       		  vectorCA[1]*vectorCA[1] + 
						   		  vectorCA[2]*vectorCA[2]);
								  
			magCAMinusRadii = (float) java.lang.Math.sqrt((double) magnitudeSquaredCA)
							  - (template.getRadius() + collidingRadius);
			magCAMinusRadii *= magCAMinusRadii;
			
			// Terminate the algorithm if the particle being considered is too far away to
			// be involved in a collision.					  
			if (magCAMinusRadii < magnitudeSquaredCB)
			{
				cosSquaredAlpha = (vectorCA[0]*vectorCB[0] + 
							       vectorCA[1]*vectorCB[1] + 
								   vectorCA[2]*vectorCB[2]);
			
				// If the numerator is not greater than zero then we do not need to 
				// consider the particle as it is on the opposite side of the motion 
				// (>90 degrees)				   
				if (cosSquaredAlpha > 0)
				{										  
					cosSquaredAlpha *= cosSquaredAlpha;
									   		
					cosSquaredAlpha /= magnitudeSquaredCB;
				
					perpendicularDistanceSquared = cosSquaredAlpha + magnitudeSquaredCA;
				
					radiiSquared = (template.getRadius() + collidingRadius);
					radiiSquared *= radiiSquared;
				
					// Do not carry out any more calculations if the line of brownian motion
					// doesn't cut a sphere with a radius equal to the sum of the radii of the 
					// potentially colliding objects.
					if (radiiSquared > perpendicularDistanceSquared)
					{
						magnitudeSquaredMaxPerp = radiiSquared - perpendicularDistanceSquared;
					
						magnitudeMax = 
							 (float) java.lang.Math.sqrt((double)cosSquaredAlpha) - 
							 (float) java.lang.Math.sqrt((double)magnitudeSquaredMaxPerp);
						magnitudeMax *= magnitudeMax;
							 
						if (magnitudeMax < motionMagnitude)
						{
							trimRatio = (float) java.lang.Math.sqrt((double)
												(magnitudeMax/motionMagnitude));
												
							for (int j=0; j<3; j++)
							{
								vectorCB[j] *= trimRatio;
							}
							
							motionMagnitude = magnitudeMax;
						}
					}
				}
			}
		}	
		for (int j=0; j<3; j++)
		{
			position[j] -= vectorCB[j];
		}
	}
	
	public void calculateElectrostaticDisplacement(float timestep, float[] displacement)
	{
		// Terminate early if this object has no charge
		if (template.getCharge() == 0f) {return;}
	
		final float electrostaticConstant = 8.99f * (float) java.lang.Math.pow(10f, 9f);
		//final float boltzmannConstant = 1.381f * (float) java.lang.Math.pow(10.0, -23.0);
		final float PI = (float) java.lang.Math.PI; 
		Particle interactionParticle;
		float endCharge;
		float endRadius;
		float[] endPosition = new float[3];
		float disp;
		float simSize;
		float[] diff = new float[3];
		float electrostaticForce;
		float temperature;
		float viscosity;
		float scaleFactor;
		float dist;
	
		for (int j=0; j<interactionList.size(); j++)
		{
			interactionParticle = (Particle) interactionList.elementAt(j);
			endCharge = interactionParticle.getCharge();
			
			// terminate early if there is no charge on the other object
			if (endCharge != 0f)
			{
				endPosition = interactionParticle.getPosition();
				endRadius = interactionParticle.getRadius();
		
				// calculate the x, y and z difference between the two vectors.
				for (int i=0; i<3; i++)
				{
					//diff[i] = position[i] - endPosition[i];
					diff[i] = boundary.dist(endPosition[i], position[i]);
				}
				dist = (float) java.lang.Math.sqrt(diff[0] * diff[0] + 
							 					 	diff[1] * diff[1] + 
							 						diff[2] * diff[2]);
	
				temperature = medium.getTemperatureK();
				viscosity = medium.getViscosity();
				
				// calculate the displacement
				if (dist != 0f)
				{
					// calculate the force
					electrostaticForce = electrostaticConstant * template.getCharge() *
										 endCharge
											/ (dist*dist);
										
					disp = (1 / (6 * PI * viscosity * template.getRadius())) 
							* electrostaticForce * timestep;
											
				}
				else
				{
					electrostaticForce = 0f;
					disp = 0f;
				}	
				
				// Check if they have met within this frame. If so set them as touching
				// - This prevents them passing through one another
				if (disp > dist*endRadius/(endRadius + template.getRadius()) -
								 template.getRadius())
				{
					disp = dist*endRadius/(endRadius + template.getRadius()) -
								 template.getRadius();
				}
				else if (-disp > dist*endRadius/(endRadius + template.getRadius()) -
								 template.getRadius())
				{
					disp = -(dist*endRadius/(endRadius + template.getRadius()) -
								 template.getRadius());
				}
		
				if (dist != 0f)
				{
					scaleFactor = disp / dist;
				}
				else 
				{
					scaleFactor = 0f;
				}
		
				for (int i=0; i<3; i++)
				{
					displacement[i] -= diff[i] * scaleFactor;
				}
			}
		}
	}
	
	public void calculateHingedDisplacement(float timestep, float[] displacement)
	{		
		final float PI = (float) java.lang.Math.PI; 
		float temperature;
		float viscosity;
		Hinge hinge;
		Particle centreParticle;
		Particle otherParticle;
		float hingeForce;
		float[] centrePos;
		float[] otherPos;
		float centreRad;
		float otherRad;
		Vector3f u;
		Vector3f v;
		Vector3f n;
		Vector3f m;
		float disp;
		float[] sep;
		
		temperature = medium.getTemperatureK();
		viscosity = medium.getViscosity();
		
		for (int j=0; j<hingeList.size(); j++)
		{
			hinge = (Hinge) hingeList.elementAt(j);
			centreParticle = hinge.getCentreParticle();
			otherParticle = hinge.getOtherParticle();
			hingeForce = hinge.getForce();
			centrePos = centreParticle.getPosition();
			otherPos = otherParticle.getPosition();
			// Calculate the a unit vector in the displacement direction
			sep = boundary.vectorBetween(centrePos, otherPos);
			u = new Vector3f(sep[0], sep[1], sep[2]);
			sep = boundary.vectorBetween(centrePos, position);
			v = new Vector3f(sep[0], sep[1], sep[2]);
			
			v.normalize();		
			u.normalize();		 
			n = new Vector3f();
			n.cross(u, v);
			n.normalize();
			m = new Vector3f();
			m.cross(v, n);
			m.normalize();
			
			Float NaNTest = new Float(m.x);
			if (NaNTest.isNaN())
			{ return; }
			NaNTest = new Float(m.y);
			if (NaNTest.isNaN())
			{ return; }
			NaNTest = new Float(m.z);
			if (NaNTest.isNaN())
			{ return; }
			
			// Calculate the displacement
			disp = (1 / (6 * PI * viscosity * template.getRadius())) * hingeForce * timestep;
			
			displacement[0] -= m.x * disp;
			displacement[1] -= m.y * disp;
			displacement[2] -= m.z * disp;
		}
	}
	
	public Vector getRandomList(int num, Random r)
	{
		// Create a randomly ordered list of particles
		Vector orderedParticles = new Vector(0);
		for (int i=0; i<num; i++)
		{
			orderedParticles.addElement(new Integer(i));
		}
		Vector randomParticles = new Vector(0);
		Integer nextParticle;
		int randInt;
		
		for (int i=0; i<num; i++)
		{
			randInt = r.nextInt(num-i);
			nextParticle = (Integer) orderedParticles.elementAt(randInt);
			orderedParticles.removeElementAt(randInt);
			randomParticles.addElement(nextParticle);
		}
		return randomParticles;
	}
	
	////////////////////////////
	// Physical attribute methods
	
	public void setPosition(float[] pos)
	{
		position = pos;
	}
	
	public float[] getPosition()
	{
		return position;
	}
	
	public float getRadius()
	{
		return template.getRadius();
	}
	
	public float getCharge()
	{
		return template.getCharge();
	}
	
	public void setParticleTemplate(ParticleTemplate pt)
	{
		template = pt;
	}
	
	public ParticleTemplate getParticleTemplate()
	{
		return template;
	}
	
	//////////////////////////
	// Medium methods
	
	public void setMedium(Medium m)
	{
		medium = m;
	}
	
	public Medium getMedium()
	{
		return medium;
	}
	
	public void setBoundary(Boundary b)
	{
		boundary = b;
	}
	
	public Boundary getBoundary()
	{
		return boundary;
	}
	/////////////////////////
	// interaction methods
	
	public void hinged(Particle centreParticle, Particle otherParticle, float force)
	{
		hingeList.addElement(new Hinge(centreParticle, otherParticle, force));
	}
	
	public void setInteractionList(Vector il)
	{
		interactionList = il;
	}

	public void interactsWith(Particle interact)
	{
		interactionList.addElement(interact);
	}
		
	public void setCollisionList(Vector cl)
	{
		collisionList = cl;
	}

	public void setConnectionList(Vector cl)
	{
		connectionList = cl;
	}
	
	public Vector getSameStructureParticles()
	{
		Vector structParts = new Vector();
		
		Connection c;
		for (int i=0; i<connectionList.size(); i++)
		{
			c = (Connection) connectionList.elementAt(i);
			structParts.addElement(c.getParticle());
		}
		
		Hinge h;
		for (int i=0; i<hingeList.size(); i++)
		{
			h = (Hinge) hingeList.elementAt(i);
			structParts.addElement(h.getCentreParticle());
			structParts.addElement(h.getOtherParticle());
		}
		
		return structParts;
	}
	
	public void collidesWith(Particle collide)
	{
		collisionList.addElement(collide);
	}

	public void connectTo(Particle connect, float dist)
	{
		connectionList.addElement(new Connection(connect, dist));
	}
	
	public void clearLists()
	{
		connectionList.removeAllElements();
		hingeList.removeAllElements();
		collisionList.removeAllElements();
		interactionList.removeAllElements();
	}
}