import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import CoordinateLayout.*;

public class ParticleTemplatePanel extends JPanel
{
	SimulationGenerator simGen;
	ParticleTemplateList ptList;
	PTNameTextField nameField;
	PTRadiusTextField radVal;
	PTRadiusUnitMenu radUnit;
	PTChargeTextField chargeVal;
	PTChargeUnitMenu chargeUnit;
	ColourSelector colour;
	ParticleTemplateStateList states;
	PTNewStateField newStateName;
	ParticleTemplateAddStateButton addStateButton;
	ParticleTemplateApplyButton applyButton;
	
	public ParticleTemplatePanel(SimulationGenerator sgen)
	{
		super();
		
		simGen = sgen; 
		
		setLayout(new BorderLayout());
		JPanel panel = new JPanel();
		panel.setPreferredSize(new Dimension(800, 200));
		panel.setLayout(new CoordinateLayout(new Dimension(25,8),4,4));
		
		addStateButton = new ParticleTemplateAddStateButton(this);
		
		// state list
		states = new ParticleTemplateStateList(simGen, this);
		
		// New state field
		newStateName = new PTNewStateField();
		
		// Name text field
		nameField = new PTNameTextField();
		
		// Radius fields
		radVal = new PTRadiusTextField();
		radUnit = new PTRadiusUnitMenu();
		
		// charge fields
		chargeVal = new PTChargeTextField();
		chargeUnit = new PTChargeUnitMenu();
		
		// Colour selector
		colour = new ColourSelector();
		
		// Apply button
		applyButton = new ParticleTemplateApplyButton(this, simGen);
				
		// Particle Template list
		ptList = 
			new ParticleTemplateList(simGen, panel, nameField, applyButton, this);
		
		JList partList = ptList.getList();
		JScrollPane scrollList = new JScrollPane(partList);
		panel.add(scrollList, new Coordinate(1,1,8,6));
		panel.add(new JLabel("Particle templates:"), new Coordinate(1,0,8,1));
		NewParticleTemplateButton tempButton = new NewParticleTemplateButton(simGen, ptList);
		panel.add(tempButton, new Coordinate(1,6,8,7));
		
		panel.add(new JLabel("Name:"), new Coordinate(9,2,12,3));
		panel.add(nameField, new Coordinate(11,2,16,3));
		
		panel.add(new JLabel("Radius:"), new Coordinate(9,3,11,4));
		panel.add(radVal, new Coordinate(11,3,13,4));
		panel.add(radUnit, new Coordinate(13,3,16,4));
		
		panel.add(new JLabel("Charge:"), new Coordinate(9,4,11,5));
		panel.add(chargeVal, new Coordinate(11,4,13,5));
		panel.add(chargeUnit, new Coordinate(13,4,16,5));
		
		panel.add(new JLabel("Colour:"), new Coordinate(9,5,11,6));
		panel.add(colour, new Coordinate(11,5,16,6));
		
		panel.add(new JLabel("States:"), new Coordinate(17,2,19,3));
		panel.add(states, new Coordinate(17,3,22,5));
		panel.add(newStateName, new Coordinate(17,5,20,6));
		panel.add(addStateButton, new Coordinate(20,5,22,6));

		
		
		panel.add(applyButton, new Coordinate(9,6,16,7));

		add(panel, BorderLayout.WEST);
	}
	
	public void resetIndex()
	{
		ptList.getList().setSelectedIndex(-1);
	}
	
	public void refresh()
	{
		if ((simGen.getParticleTemplates()).size() >= 1)
		{
			ptList.getList().setSelectedIndex(0);
			nameField.enableField();
			applyButton.setEnabled(true);
			getAddStateButton().setEnabled(true);
			
			getRadField().enableField();
			getRadUnit().enableField();
			
			getChargeField().enableField();
			getChargeUnit().enableField();
			
			getColourSelector().enableField();
			
			getStateList().refresh();
			
			getNewStateName().enableField();
			
		}
		else
		{
			nameField.disableField();
			applyButton.setEnabled(false);
			getAddStateButton().setEnabled(false);
			
			getRadField().disableField();
			getRadUnit().disableField();
			
			getChargeField().disableField();
			getChargeUnit().disableField();
			
			getColourSelector().disableField();
			getStateList().disableField();
			
			getNewStateName().disableField();
		}
		ptList.refresh();
	}
	
	public ParticleTemplateAddStateButton getAddStateButton()
	{
		return addStateButton;
	}
	
	public ParticleTemplate currentTemplate()
	{
		return ptList.getCurrentTemplate();
	}
	
	public ParticleTemplateList getList()
	{
		return ptList;
	}
	
	public SimulationGenerator getSimGen()
	{
		return simGen;
	}
	
	public PTNameTextField getNameField()
	{
		return nameField;
	}
	
	public PTRadiusTextField getRadField()
	{
		return radVal;
	}
	
	public PTRadiusUnitMenu getRadUnit()
	{
		return radUnit;
	}
	
	public PTChargeTextField getChargeField()
	{
		return chargeVal;
	}
	
	public PTChargeUnitMenu getChargeUnit()
	{
		return chargeUnit;
	}
	
	public ColourSelector getColourSelector()
	{
		return colour;
	}
	
	public ParticleTemplateStateList getStateList()
	{
		return states;
	}
	
	public PTNewStateField getNewStateName()
	{
		return newStateName;
	}
}

// addState button
class ParticleTemplateAddStateButton extends JButton implements ActionListener
{
	ParticleTemplatePanel panel;
	
	public ParticleTemplateAddStateButton(ParticleTemplatePanel p)
	{
		super("Add");
		panel = p;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		Vector pt = panel.getSimGen().getParticleTemplates();
		ParticleTemplate ptemp = (ParticleTemplate) 
				(pt.elementAt(panel.getList().getList().getSelectedIndex()));
		
		String newState = panel.getNewStateName().getVal();
		if (!(newState.equals("")))
		{
			ptemp.getStates().addElement(newState);
			panel.getStateList().refresh();
			panel.getNewStateName().setVal("");
		}
	}
}

class PTNewStateField extends JTextField
{
	String val;
	
	public PTNewStateField()
	{
		super(6);
		disableField();
		setHorizontalAlignment(JTextField.RIGHT);
	}
	
	public void disableField()
	{
		setText("");
		setEnabled(false);
	}
	
	public void enableField()
	{
		setEnabled(true);
	}
	
	public void setVal(String n)
	{
		setText(n);
	}
	
	public String getVal()
	{
		return getText();
	}
}

class ParticleTemplateStateList extends JPanel
{
	SimulationGenerator simGen;
	JList stateList;
	ParticleTemplatePanel panel;
	
	public ParticleTemplateStateList(SimulationGenerator sgen, ParticleTemplatePanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		stateList = new JList();
		stateList.setVisibleRowCount(2);
		stateList.setFixedCellWidth(150);
		stateList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		stateList.setEnabled(true);
		stateList.setSelectionForeground(new Color(0,0,0));
		stateList.setSelectionBackground(new Color(255,255,255));
		JScrollPane scrollPane = new JScrollPane(stateList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}
	
	public void refresh()
	{		
		if (!(panel.getList() == null))
		{
			int tempNo = panel.getList().getList().getSelectedIndex();
		
			ParticleTemplate temp = (ParticleTemplate) 
							simGen.getParticleTemplates().elementAt(tempNo);
			Vector states = temp.getStates();
		
			Vector names = new Vector();
			for (int i=0; i<states.size(); i++)
			{
				names.add(states.elementAt(i));
			}
			int select = stateList.getSelectedIndex();
			stateList.setListData(names);
			stateList.setSelectedIndex(select);
		}
	}
	
	public void disableField()
	{
		stateList.setListData(new Vector());
	}
	
	public JList getList()
	{
		return stateList;
	}
}

class ColourSelector extends JPanel implements ActionListener
{
	JButton change;
	JTextField swatch;
	Color colour;
	
	public ColourSelector()
	{
		super();

		setLayout(new CoordinateLayout(new Dimension(8,1), 4, 4));
		
		colour = new Color(255,255,255);
		swatch = new JTextField();
		swatch.setEnabled(false);
		swatch.setBackground(colour);
		add(swatch, new Coordinate(0,0,4,1));
		
		change = new JButton("change");
		change.addActionListener(this);
		add(change, new Coordinate(4,0,8,1));
	}
	
	public void actionPerformed(ActionEvent e)
	{
		Color old = colour;
		colour = JColorChooser.showDialog(this, "Particle Colour", colour);
		if (colour == null)
		{
			colour = old;
		}
		
		swatch.setBackground(colour);
		swatch.paintImmediately(swatch.getVisibleRect());
	
	}
	
	public Color getColour()
	{
		return colour;
	}
	
	public void setColour(Color c)
	{
		colour = c;
		swatch.setBackground(colour);
	}
	
	public void disableField()
	{
		colour = new Color(255,255,255);
		swatch.setBackground(colour);
		change.setEnabled(false);
	}
	
	public void enableField()
	{
		change.setEnabled(true);
	}
}

class PTChargeTextField extends JTextField implements KeyListener
{
	String val;
	
	public PTChargeTextField()
	{
		super(6);
		disableField();
		
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}
	
	public void disableField()
	{
		setText("");
		val="";
		setEnabled(false);
	}
	
	public void enableField()
	{
		setEnabled(true);
	}
		
	public void keyPressed(KeyEvent ke)
	{}
	
	public void keyTyped(KeyEvent ke)
	{}
	
	public void keyReleased(KeyEvent ke)
	{
		try 
		{
			val = "" + (float) (Float.parseFloat(getText()));
		}
		catch (NumberFormatException e)
		{}
	}
	
	public void setVal(float v)
	{
		val = v + "";
		setText(val);
	}
	
	public float getVal()
	{
		try 
		{
			return (float) (Float.parseFloat(val));
		}
		catch (NumberFormatException e)
		{}
		
		return 0f;
	}
}

class PTChargeUnitMenu extends JComboBox implements ItemListener
{
	float radUnit;
	
	public PTChargeUnitMenu()
	{
		super();
		
		addItem("Coloumbs");
		addItem("Millicoloumbs");
		addItem("Microcoloumbs");
		addItem("Nanocoloumbs");
		addItem("Picocoloumbs");
		addItem("Femtocoloumbs");
		addItem("Attocoloumbs");
		
		setEditable(false);
	
		addItemListener(this);
	}
	
	public float getUnit()
	{
		return radUnit;
	}
	
	public void setUnit(float unit)
	{
		radUnit = unit;
		
		if (unit < 1.1f && unit > 0.9f) { setSelectedIndex(0); }
		if (unit < (float) Math.pow(10.0, -2.0) && unit > (float) Math.pow(10.0, -4.0)) 
			{ setSelectedIndex(1); }
		if (unit < (float) Math.pow(10.0, -5.0) && unit > (float) Math.pow(10.0, -7.0)) 
			{ setSelectedIndex(2); }
		if (unit < (float) Math.pow(10.0, -8.0) && unit > (float) Math.pow(10.0, -10.0)) 
			{ setSelectedIndex(3); }
		if (unit < (float) Math.pow(10.0, -11.0) && unit > (float) Math.pow(10.0, -13.0)) 
			{ setSelectedIndex(4); }
		if (unit < (float) Math.pow(10.0, -14.0) && unit > (float) Math.pow(10.0, -16.0)) 
			{ setSelectedIndex(5); }
		if (unit < (float) Math.pow(10.0, -17.0) && unit > (float) Math.pow(10.0, -19.0)) 
			{ setSelectedIndex(6); }
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();
			
		if (s.equals("Coloumbs")) { radUnit = 1f;  }
		if (s.equals("Millicoloumbs")) 
			{ radUnit = (float) Math.pow(10.0, -3.0);  }
		if (s.equals("Microcoloumbs")) 
			{ radUnit = (float) Math.pow(10.0, -6.0);  }
		if (s.equals("Nanocoloumbs")) 
			{ radUnit = (float) Math.pow(10.0, -9.0);  }
		if (s.equals("Picocoloumbs")) 
			{ radUnit = (float) Math.pow(10.0, -12.0);  }
		if (s.equals("Femtocoloumbs")) 
			{ radUnit = (float) Math.pow(10.0, -15.0);  }
		if (s.equals("Attocoloumbs")) 
			{ radUnit = (float) Math.pow(10.0, -18.0);  }
	}
	
	public void enableField()
	{ 
		removeItem("");
		setEnabled(true);
	}
	
	public void disableField()
	{
		addItem("");
		setSelectedItem("");
		setEnabled(false);
	}
}

class PTRadiusTextField extends JTextField implements KeyListener
{
	String val;
	
	public PTRadiusTextField()
	{
		super(6);
		disableField();
		
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}
	
	public void disableField()
	{
		setText("");
		val="";
		setEnabled(false);
	}
	
	public void enableField()
	{
		setEnabled(true);
	}
		
	public void keyPressed(KeyEvent ke)
	{}
	
	public void keyTyped(KeyEvent ke)
	{}
	
	public void keyReleased(KeyEvent ke)
	{
		try 
		{
			val = "" + (float) (Float.parseFloat(getText()));
		}
		catch (NumberFormatException e)
		{}
	}
	
	public void setVal(float v)
	{
		val = v + "";
		setText(val);
	}
	
	public float getVal()
	{
		try 
		{
			return (float) (Float.parseFloat(val));
		}
		catch (NumberFormatException e)
		{}
		
		return 0f;
	}
}

class PTRadiusUnitMenu extends JComboBox implements ItemListener
{
	float radUnit;
	
	public PTRadiusUnitMenu()
	{
		super();
		
		addItem("Metres");
		addItem("Millimetres");
		addItem("Micrometres");
		addItem("Nanometres");
		addItem("Picometres");
		addItem("Femtometres");
		
		setEditable(false);
	
		addItemListener(this);
	}
	
	public float getUnit()
	{
		return radUnit;
	}
	
	public void setUnit(float unit)
	{
		radUnit = unit;
		
		if (unit < 1.1f && unit > 0.9f) { setSelectedIndex(0); }
		if (unit < (float) Math.pow(10.0, -2.0) && unit > (float) Math.pow(10.0, -4.0)) 
			{ setSelectedIndex(1); }
		if (unit < (float) Math.pow(10.0, -5.0) && unit > (float) Math.pow(10.0, -7.0)) 
			{ setSelectedIndex(2); }
		if (unit < (float) Math.pow(10.0, -8.0) && unit > (float) Math.pow(10.0, -10.0)) 
			{ setSelectedIndex(3); }
		if (unit < (float) Math.pow(10.0, -11.0) && unit > (float) Math.pow(10.0, -13.0)) 
			{ setSelectedIndex(4); }
		if (unit < (float) Math.pow(10.0, -14.0) && unit > (float) Math.pow(10.0, -16.0)) 
			{ setSelectedIndex(5); }
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();
		
		if (s.equals("Metres")) { radUnit = 1f;  }
		if (s.equals("Millimetres")) 
			{ radUnit = (float) Math.pow(10.0, -3.0);  }
		if (s.equals("Micrometres")) 
			{ radUnit = (float) Math.pow(10.0, -6.0);  }
		if (s.equals("Nanometres")) 
			{ radUnit = (float) Math.pow(10.0, -9.0);  }
		if (s.equals("Picometres")) 
			{ radUnit = (float) Math.pow(10.0, -12.0);  }
		if (s.equals("Femtometres")) 
			{ radUnit = (float) Math.pow(10.0, -15.0);  }	
	}
	
	public void enableField()
	{ 
		removeItem("");
		setEnabled(true);
	}
	
	public void disableField()
	{
		addItem("");
		setSelectedItem("");
		setEnabled(false);
	}
}

class ParticleTemplateList
{
	SimulationGenerator simGen;
	JPanel panel;
	JList templateList;
	PTNameTextField name;
	ParticleTemplateApplyButton applyButton;
	ParticleTemplatePanel main;
	
	public ParticleTemplateList(SimulationGenerator sgen, JPanel p, PTNameTextField ptnf,
								ParticleTemplateApplyButton app, ParticleTemplatePanel m)
	{
		simGen = sgen;
		panel = p;
		name = ptnf;
		applyButton = app;
		main = m;
		
		templateList = new JList();
		templateList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		if ((simGen.getParticleTemplates()).size() >= 1)
		{
			templateList.setSelectedIndex(0);
			name.enableField();
			applyButton.setEnabled(true);
			main.getAddStateButton().setEnabled(true);
			
			main.getRadField().enableField();
			main.getRadUnit().enableField();
			
			main.getChargeField().enableField();
			main.getChargeUnit().enableField();
			
			main.getColourSelector().enableField();
			
			main.getStateList().refresh();
			
			main.getNewStateName().enableField();
			
		}
		else
		{
			name.disableField();
			applyButton.setEnabled(false);
			main.getAddStateButton().setEnabled(false);
			
			main.getRadField().disableField();
			main.getRadUnit().disableField();
			
			main.getChargeField().disableField();
			main.getChargeUnit().disableField();
			
			main.getColourSelector().disableField();
			main.getStateList().disableField();
			
			main.getNewStateName().disableField();
		}
		
		templateList.addListSelectionListener(
			new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e)
				{
					int select = templateList.getSelectedIndex();
					if (simGen.getParticleTemplates().size()<=select)
					{ select = -1; }
					
					if (select == -1)
					{
						name.disableField();
						applyButton.setEnabled(false);
						main.getAddStateButton().setEnabled(false);
						
						
						main.getRadField().disableField();
						main.getRadUnit().disableField();
						
						main.getChargeField().disableField();
						main.getChargeUnit().disableField();
						
						main.getColourSelector().disableField();
						
						main.getStateList().disableField();
						
						main.getNewStateName().disableField();
					}
					else 
					{
						ParticleTemplate ptemp = ((ParticleTemplate) 
								(simGen.getParticleTemplates()).elementAt(select));
						name.setVal(ptemp.getName());
						name.enableField();
						applyButton.setEnabled(true);
						main.getAddStateButton().setEnabled(true);
						
						main.getRadField().setVal(ptemp.getRadiusVal());
						main.getRadField().enableField();
						main.getRadUnit().setUnit(ptemp.getRadiusUnit());
						main.getRadUnit().enableField();
						
						main.getChargeField().setVal(ptemp.getChargeVal());
						main.getChargeField().enableField();
						main.getChargeUnit().setUnit(ptemp.getChargeUnit());
						main.getChargeUnit().enableField();
						
						main.getColourSelector().setColour(ptemp.getColour());
						main.getColourSelector().enableField();
						
						main.getNewStateName().enableField();
						
						main.getStateList().refresh();
					}
				}
			}
		);
		
		refresh();
	}
	
	public void refresh()
	{		
		Vector particleTemplates = simGen.getParticleTemplates();
		Vector names = new Vector();
		for (int i=0; i<particleTemplates.size(); i++)
		{
			ParticleTemplate pt = (ParticleTemplate) (particleTemplates.elementAt(i));
			names.add(pt.getName());
		}
		int select = templateList.getSelectedIndex();
		templateList.setListData(names);
		templateList.setSelectedIndex(select);
	}
	
	public JList getList()
	{
		return templateList;
	}
	
	public ParticleTemplate getCurrentTemplate()
	{
		return (ParticleTemplate) 
				(simGen.getParticleTemplates().elementAt(templateList.getSelectedIndex()));
	}
}

// new template button
class NewParticleTemplateButton extends JButton implements ActionListener
{
	SimulationGenerator simGen;
	ParticleTemplateList partTempList;
	
	public NewParticleTemplateButton(SimulationGenerator sg, ParticleTemplateList ptl)
	{
		super("New");
		addActionListener(this);
		simGen = sg;
		partTempList = ptl;
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		Vector pt = simGen.getParticleTemplates();
		new ParticleTemplate(pt);
		partTempList.refresh();
		(partTempList.getList()).setSelectedIndex(simGen.getParticleTemplates().size()-1);
	}
}

// apply button
class ParticleTemplateApplyButton extends JButton implements ActionListener
{
	ParticleTemplatePanel panel;
	SimulationGenerator simGen;
	
	public ParticleTemplateApplyButton(ParticleTemplatePanel p, SimulationGenerator sg)
	{
		super("Apply Changes");
		panel = p;
		simGen = sg;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		Vector pt = panel.getSimGen().getParticleTemplates();
		ParticleTemplate ptemp = (ParticleTemplate) 
				(pt.elementAt(panel.getList().getList().getSelectedIndex()));
		ptemp.setName(panel.getNameField().getVal());
		
		ptemp.setRadiusVal(panel.getRadField().getVal());
		ptemp.setRadiusUnit(panel.getRadUnit().getUnit());
		
		ptemp.setChargeVal(panel.getChargeField().getVal());
		ptemp.setChargeUnit(panel.getChargeUnit().getUnit());
		
		ptemp.setColour(panel.getColourSelector().getColour());
		
		panel.getList().refresh();
		
		simGen.updateSimulation();
	}
}

class PTNameTextField extends JTextField implements KeyListener
{
	String val;
	
	public PTNameTextField()
	{
		super(6);
		disableField();
		
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}
	
	public void disableField()
	{
		setText("");
		setEnabled(false);
	}
	
	public void enableField()
	{
		setEnabled(true);
	}
		
	public void keyPressed(KeyEvent ke)
	{}
	
	public void keyTyped(KeyEvent ke)
	{}
	
	public void keyReleased(KeyEvent ke)
	{
		val = getText();
	}
	
	public void setVal(String v)
	{
		val = v;
		setText(val);
	}
	
	public String getVal()
	{
		return val;
	}
}
