import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
 
public class SimulationCanvas extends JPanel
{
	Boundary boundary;
	SimulationGroup simulationObjects;
	SimulationGenerator simulationGenerator;
	
	public SimulationCanvas(String[] args) 
	{		
		// Set initial viewpoint at 20m. Note that the simulation will 
		// always be scaled to fit in a 10m cubed cube
		Vector3d viewVector = new Vector3d(0f, 0f, 30f);
		
		// This should be large enough to encompass the bounding box fully
		float boundSphereRadius = 10000.0f; 
	
		// Create a canvas for the simulation //
	    Canvas3D canvas3D = new Canvas3D(new GraphConfig().getGraphConfig());
    	this.setLayout(new BorderLayout());
  	 	this.setOpaque(false);
   	 	this.add("Center", canvas3D);
		
		// Create the universe and viewing platform //
		VirtualUniverse universe = new VirtualUniverse( );
        Locale locale = new Locale(universe);
		
		PhysicalBody body = new PhysicalBody( );
		PhysicalEnvironment environment = new PhysicalEnvironment( );
		View view = new View( );
		view.addCanvas3D(canvas3D);
		view.setPhysicalBody(body);
		view.setPhysicalEnvironment(environment);

		BranchGroup viewRoot = new BranchGroup( );
		TransformGroup viewTransform = new TransformGroup( );
		viewTransform.setCapability( TransformGroup.ALLOW_TRANSFORM_READ );
		viewTransform.setCapability( TransformGroup.ALLOW_TRANSFORM_WRITE );
		Transform3D transformvp = new Transform3D( );
		transformvp.set(viewVector);
		viewTransform.setTransform(transformvp);
		viewRoot.insertChild(viewTransform, 0);
    
		ViewPlatform vp = new ViewPlatform();
		
		view.attachViewPlatform( vp );
		viewTransform.addChild( vp );

		locale.addBranchGraph( viewRoot );
		
		// 1. Create the root group for the simulation //
		
	   	BranchGroup rootGroup = new BranchGroup();
		
		  // 1.1. Create a lighting node //
		  
		  Color3f lightColor = new Color3f(1.4f, 1.4f, 1.4f);
   	   	  BoundingSphere bounds =
    	  	   new BoundingSphere(new Point3d(0.0, 0.0, 0.0), boundSphereRadius);
			   
	      Point3f lightSource;
   	   	  Point3f atten;
		  PointLight light;
		  
		  lightSource = new Point3f(0f, 0f, 30f);
		  atten = new Point3f(0f, 0.025f, 0f);
   	   	  light = new PointLight(lightColor, lightSource, atten);
   	   	  light.setInfluencingBounds(bounds);
   	   	  rootGroup.addChild(light);
		  
		  lightColor = new Color3f(0.7f, 0.7f, 0.7f);
		  lightSource = new Point3f(0f, 0f, -15f);
		  atten = new Point3f(1f, 0.03f, 0f);
   	   	  light = new PointLight(lightColor, lightSource, atten);
   	   	  light.setInfluencingBounds(bounds);
   	   	  rootGroup.addChild(light);
		  
		  AmbientLight amb = new AmbientLight(new Color3f(0.6f, 0.6f, 0.6f));
		  amb.setInfluencingBounds(bounds);
		  rootGroup.addChild(amb);

		  // 1.2. Create the simulation transform group //
		
		  TransformGroup simulationGroup = new TransformGroup();
		
		  simulationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    	  simulationGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		  simulationGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
      	  simulationGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    	  simulationGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);	
		  
		  MouseRotate mouseRotate = new MouseRotate(simulationGroup);
   		  mouseRotate.setSchedulingBounds(bounds);
    	  simulationGroup.addChild(mouseRotate);
		  
		  MouseZoom mouseZoom = new MouseZoom(simulationGroup);
   		  mouseZoom.setSchedulingBounds(new BoundingSphere());
    	  simulationGroup.addChild(mouseZoom);

			// 1.2.1. Create the bounding box //
				
			boundary = new Boundary("tessellated",100f * (float) java.lang.Math.pow(10f, -9f));
			Group boundBox = new Group();		
			boundary.drawBox(boundBox);
			//boundBox.addChild(boundary.getBox());
			
			simulationGroup.addChild(boundBox);
			
			simulationGenerator = new SimulationGenerator(simulationGroup, args);
			
			/*
			// Create a group for the simulation objects
			simulationObjects = new SimulationGroup();

		  	simulationObjects.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
    	  	simulationObjects.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    	  	simulationObjects.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);	  
		
			simulationGroup.addChild(simulationObjects);
			*/
			
			// End 1.2.1.
			
    	  rootGroup.addChild(simulationGroup);	
		  
		  // End 1.2.
		  
 		locale.addBranchGraph(rootGroup);
		
		// End Scenegraph
	}
	
	public SimulationGroup getSimulationGroup()
	{
		return simulationGenerator.getSimulationGroup();
	}
	
	public SimulationGenerator getSimulationGenerator()
	{
		return simulationGenerator;
	}
}

class GraphConfig
{
	GraphicsConfiguration getGraphConfig()
	{
		GraphicsConfigTemplate3D tmpl = new GraphicsConfigTemplate3D( ); 
		GraphicsEnvironment env =
				GraphicsEnvironment.getLocalGraphicsEnvironment( ); 
		GraphicsDevice device = env.getDefaultScreenDevice( ); 
		GraphicsConfiguration config = device.getBestConfiguration( tmpl ); 
		return config;
	}
};