import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

public class SimulationGenerator
{
	TransformGroup mainGroup;
	SimulationGroup simGroup;
	Medium medium;
	Boundary boundary;
	Vector particleTemplates;
	Vector objects;
	Vector rootObjects;
	Vector objectTemplates;

	Vector objectInstances;
	Vector objectInstancesPos;

	ControlPanel controlPanel = null;

	String[] args;

	float boundSize;
	int temperatureUnit;
	final int CELCIUS = 0;
	final int KELVIN = 0;
	float viscosityVal;
	float viscosityUnit;

	final float nano = (float) Math.pow(10.0, -9.0);

        // Create a variable to hold the current algorithm
        final int DISSERTATION_ALGORITHM = 0;
        final int MONTE_CARLO = 1;
        final int DISSIPATIVE_PARTICLE_DYNAMICS = 2;
        int updateAlgorithm = DISSERTATION_ALGORITHM;

	public SimulationGenerator(TransformGroup mg, String[] a)
	{
		mainGroup = mg;
		args = a;

                // determine the correct algorithm based on the command line arguments
                for (int i=0; i<args.length; i++)
                {
                        if (args[i].equals("-Adissertation"))
                        {
                                updateAlgorithm = DISSERTATION_ALGORITHM;
                        }
                        else if (args[i].equals("-Amontecarlo"))
                        {
                                updateAlgorithm = MONTE_CARLO;
                        }
                        else if (args[i].equals("-Adissipative"))
                        {
                                updateAlgorithm = DISSIPATIVE_PARTICLE_DYNAMICS;
                        }
                }

		// Set up the boundary and medium
		boundary = new Boundary("tessellated", 500f * (float) Math.pow(10.0, -9.0));
		medium = new Medium();

		temperatureUnit = CELCIUS;
		setTemperature(20f);
		viscosityVal = 1f;
		viscosityUnit = (float) Math.pow(10.0, -9.0);
		medium.setViscosity(viscosityVal * viscosityUnit);

		objects = new Vector();
		rootObjects = new Vector();
		particleTemplates = new Vector();
		objectTemplates = new Vector();

		objectInstances = new Vector();
		objectInstancesPos = new Vector();

		sampleApplications(args);
		createObjectInstances();

		newSimulationGroup();
		mainGroup.insertChild(simGroup, 0);
	}

	public void setControlPanel(ControlPanel cp)
	{
		controlPanel = cp;
	}

	public void newSimulation()
	{
		args = new String[0];
		objects = new Vector();
		rootObjects = new Vector();
		particleTemplates = new Vector();
		objectTemplates = new Vector();

		objectInstances = new Vector();
		objectInstancesPos = new Vector();

		createObjectInstances();
		
		updateSimulation();
		simGroup.reset(0);
	}
	
	public void addObjectInstance(ObjectTemplate oT, float[] pos)
	{
		objectInstances.addElement(oT);
		objectInstancesPos.addElement(pos);
	}
	
	public Vector getObjectTemplates()
	{
		return objectTemplates;
	}
	
	public Vector getAllObjects()
	{
		return objects;
	}
	
	public Vector getRootObjects()
	{
		return rootObjects;
	}
	
	public Vector getParticleTemplates()
	{
		return particleTemplates;
	}
	
	public void newSimulationGroup()
	{
		SimObject[] objArray = new SimObject[objects.size()];
		for (int i=0; i<objects.size(); i++)
		{
			objArray[i] = ((SimObject)(objects.elementAt(i)));
		}
		if (simGroup != null)
		{
			simGroup.enabled(false);
		}
		
		simGroup = new SimulationGroup(boundary, medium, objArray, particleTemplates, args, updateAlgorithm);
		simGroup.setCapability(BranchGroup.ALLOW_DETACH);

		simGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
    	simGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    	simGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
	}
	
	public void setViscosityValue(float v)
	{
		viscosityVal = v;
		medium.setViscosity(viscosityVal * viscosityUnit);
	}
	
	public void setViscosityUnit(float u)
	{
		viscosityUnit = u;
		medium.setViscosity(viscosityVal * viscosityUnit);
	}
	
	public void setTemperature(float t)
	{
		if (temperatureUnit == CELCIUS)
		{
			medium.setTemperatureC(t);
		}
		else 
		{
			medium.setTemperatureK(t);
		}
	}
	
	public void setTemperatureUnit(int unit)
	{
		temperatureUnit = unit;
	}
		
	public void setBoundVal(float val)
	{
		removeSimulationGroup();
		boundSize = val;
		boundary.setSimulationSize(boundSize);
		newSimulationGroup();
		addSimulationGroup();
	}
	
	public void updateSimulation()
	{
		removeSimulationGroup();
		createObjectInstances();
		newSimulationGroup();
		addSimulationGroup();
		if (controlPanel != null)
		{
			controlPanel.refresh();
		}
	}
	
	public void createObjectInstances()
	{
		for (int i=0; i<particleTemplates.size(); i++)
		{
			((ParticleTemplate)(particleTemplates.elementAt(i))).setMembers(new Vector());
		}
		// Create the object instances
		objects = new Vector();
		rootObjects = new Vector();
		for (int i=0; i<objectInstances.size(); i++)
		{
			ObjectTemplate objTemp = (ObjectTemplate) (objectInstances.elementAt(i));
			float[] pos = (float[]) (objectInstancesPos.elementAt(i));
			SimObject[] obs = objTemp.createInstance(pos);
			for (int j=0; j<obs.length; j++)
			{
				objects.addElement(obs[j]);
			}
			rootObjects.addElement(obs[0]);
		}
	}
	
	public void removeSimulationGroup()
	{
		mainGroup.removeChild(0);
	}
	
	public void addSimulationGroup()
	{
		mainGroup.insertChild(simGroup, 0);
	}
	
	public SimulationGroup getSimulationGroup()
	{
		return simGroup;
	}
	
	public void sampleApplications(String[] a)
	{
		args = a;
		// -microtubule example
		if (args.length>0 && args[0].equals("-microtubule"))
		{
			int protoLength = -1;
			if (args.length>1)
			{
				try 
				{
					protoLength = (int) (Integer.parseInt(args[1]));
				}
				catch (NumberFormatException e)
				{}
			}
			sampleMicrotubule(protoLength);
		}
		
		// -transport example
		if (args.length>0 && args[0].equals("-transport"))
		{
			int protoLength = -1;
			if (args.length>1)
			{
				try 
				{
					protoLength = (int) (Integer.parseInt(args[1]));
				}
				catch (NumberFormatException e)
				{}
			}
			sampleTransport(protoLength);
		}
		
		// -particlegroup example
		if (args.length>0 && args[0].equals("-particlegroup"))
		{
			int protoLength = -1;
			if (args.length>1)
			{
				try 
				{
					protoLength = (int) (Integer.parseInt(args[1]));
				}
				catch (NumberFormatException e)
				{}
			}
			sampleParticleGroup(protoLength);
		}
		
		// -electrodimers example
		if (args.length>0 && args[0].equals("-electrodimers"))
		{
			sampleElectrodimers();
		}
	}
	
	public void sampleMicrotubule(int protoLength)
	{
		//////////////////////////////////////////////////
		/// Sample app ...
		//////////////////////////////////////////////////
		// colours
		final float[] red = {1f, 0.2f, 0.2f};
		final float[] blue = {0.2f, 0.2f, 1f};
		
		// radii
		final float alphaRadius = 2f;
		final float betaRadius = 2f;
		
		// protofilament length (in dimer)
		if (protoLength == -1)
		{
			protoLength = 10;
		}
		
		//boundary = new Boundary("tessellated", 500f * nano);
		
		// Connection lengths
		final float dimerCon = alphaRadius*nano + betaRadius*nano;
		final float protoCon = alphaRadius*nano + betaRadius*nano;
		
		// Hinge forces
		final float protoHinge = 2f* (float) Math.pow(10.0, -23.0);
		
		particleTemplates = new Vector();
		//objects = new Object[0];
		
		// alpha particle
		ParticleTemplate alpha = new ParticleTemplate(particleTemplates);
		alpha.setColour(red);
		alpha.setRadiusVal(alphaRadius);
		alpha.setRadiusUnit(nano);
		alpha.setName("Alpha tubulin");
		
		//float[] alphaPos = {0f,0f,0f};
		//float[] alphaPos2 = {4f*nano,0f,0f};
		//alpha.createInstance(alphaPos);
		//alpha.createInstance(alphaPos2);
		
		// beta particle
		ParticleTemplate beta = new ParticleTemplate(particleTemplates);
		beta.setColour(blue);
		beta.setRadiusVal(betaRadius);
		beta.setRadiusUnit(nano);
		beta.setName("Beta tubulin");
			
		// dimer
		ObjectTemplate dimer = new ObjectTemplate(objectTemplates);
		dimer.setName("Dimer");
		float[] alphaPos = {0f,0f,0f};
		float[] betaPos= {alphaRadius*nano+betaRadius*nano,0f, 0f};
		dimer.addParticle(alpha, alphaPos);
		dimer.addParticle(beta, betaPos);
			
		dimer.addConnection(0, 1, dimerCon, 0);
		
		// protofilament
		ObjectTemplate protofilament = new ObjectTemplate(objectTemplates);
		protofilament.setName("Protofilament");
		float[] dimerPos = new float[3];
		for (int i=0; i<protoLength; i++)
		{
			dimerPos[0] = (2f*alphaRadius*nano + 2f*betaRadius*nano)*i;
			dimerPos[0] -= 4f*nano*protoLength;
			dimerPos[1] = 0f;
			dimerPos[2] = 0f;
			protofilament.addObject(dimer, dimerPos);
		}
		
		for (int i=0; i<protoLength-1; i++)
		{
			protofilament.addConnection(2*i+2, 2*i+1, 4f*nano, 0);
			//protofilament.addHinge(2*i+1, 2*i+0, 2*i+2, protoHinge, 0);
			//protofilament.addHinge(2*i+2, 2*i+1, 2*i+3, protoHinge, 0);
		}
		protofilament.addConnection(0, protoLength*2-1, 8f*(protoLength-0.5f)*nano, 0);
		
		//float[] microPos = {0f, 0f, 0f};
		//objects = protofilament.createInstance(microPos);
			
		
		// microtubule
		ObjectTemplate microtubule = new ObjectTemplate(objectTemplates);
		microtubule.setName("Microtubule");
		int protos = 13;
		float[] protoPos = new float[3];
		float theta;
		for (int i=0; i<protos; i++)
		{
			theta = 27.7f * (float) i;
			protoPos[0] = i * 0.923f * nano;
			protoPos[1] = 10f * nano * (float) Math.cos((double) theta * 3.141 / 180.0);
			protoPos[2] = 10f * nano * (float) Math.sin((double) theta * 3.141 / 180.0);;
			microtubule.addObject(protofilament, protoPos);
		}
		
		// connect adjacent tubulins
		for (int i=0; i<protos-1; i++)
		{
			for (int j=0; j<protoLength*2; j++)
			{
				microtubule.
					addConnection(i*protoLength*2+j, (i+1)*protoLength*2+j, 4.88f*nano, 0);
				
				//if (j>0)
				//{
				//	microtubule.
				//		addConnection(i*protoLength*2+j, (i+1)*protoLength*2+j-1, 
				//					  6.18f*nano, 0);
					
				//}
			}
		}
			
		// join up the seam
		for (int i=3; i<(protoLength*2); i++)
		{
			microtubule.addConnection(i, (protos-1)*protoLength*2+i-3, 4.88f*nano, 0);
		}
			
		// Create rigid spokes to prevent buckling
		for (int i=0; i<protoLength*2; i++)
		{
			int current=5; 
			int old=0;
			
			for (int j=0; j<protos; j++)
			{
				microtubule.addConnection(old*protoLength*2+i, current*protoLength*2+i,
										  19.26f*nano, 0);
				if (i<protoLength*2-2)
				{
					microtubule.addConnection(old*protoLength*2+i,
					 	current*protoLength*2+i+2, 20.11f*nano, 0);
				}
									  
				old = current;
				current = (current+5)%13;
			}
		}
	
		/*
		// Create supports to prevent axial rotation
		float conDist = (float) Math.sqrt( 
							(((protoLength-1)*8f*nano+4.62f*nano) *
							 ((protoLength-1)*8f*nano+4.62f*nano)) +
							18.7f*nano*18.7f*nano);
		for (int k=0; k<20; k++)
		{
		for (int i=0; i<protos-5; i++)
		{
			microtubule.addConnection(i*protoLength*2,
							 ((i+5)*protoLength*2)+protoLength*2-1, 
							 conDist, 0);
		}
		}
		*/
	
		// Create a microtubule
		float[] microPos = {0f, 0f, 0f};
		addObjectInstance(microtubule, microPos);
	
		//////////////////////////////////////////////////
	}
	
	/*
	public void sampleTransport(int protoLength)
	{
		// colours
		final float[] red = {1f, 0.2f, 0.2f};
		final float[] blue = {0.2f, 0.2f, 1f};
		final float[] green = {0.2f, 1f, 0.2f};
		
		// radii
		final float alphaRadius = 2f;
		final float betaRadius = 2f;
		
		// protofilament length (in dimer)
		if (protoLength == -1)
		{
			protoLength = 10;
		}
		
		boundary = new Boundary("tessellated", 3.5f * 8f * protoLength * nano);
		
		// Connection lengths
		final float dimerCon = alphaRadius*nano + betaRadius*nano;
		final float protoCon = alphaRadius*nano + betaRadius*nano;
		
		// Hinge forces
		final float protoHinge = 2f* (float) Math.pow(10.0, -23.0);
		
		particleTemplates = new Vector();
		SimObject[][] objectList = new SimObject[2][];
		
		// alpha particle
		ParticleTemplate alpha = new ParticleTemplate(particleTemplates);
		alpha.setColour(red);
		alpha.setRadiusVal(alphaRadius);
		alpha.setRadiusUnit(nano);
		alpha.setName("Alpha tubulin");
		
		// beta particle
		ParticleTemplate beta = new ParticleTemplate(particleTemplates);
		beta.setColour(blue);
		beta.setRadiusVal(betaRadius);
		beta.setRadiusUnit(nano);
		beta.setName("Beta tubulin");
			
		// dimer
		ObjectTemplate dimer = new ObjectTemplate();
		float[] alphaPos = {0f,0f,0f};
		float[] betaPos= {alphaRadius*nano+betaRadius*nano,0f, 0f};
		dimer.addParticle(alpha, alphaPos);
		dimer.addParticle(beta, betaPos);
			
		dimer.addConnection(0, 1, dimerCon, 0);
		
		// protofilament
		ObjectTemplate protofilament = new ObjectTemplate();
		float[] dimerPos = new float[3];
		for (int i=0; i<protoLength; i++)
		{
			dimerPos[0] = (2f*alphaRadius*nano + 2f*betaRadius*nano)*i;
			dimerPos[0] -= 4f*nano*protoLength;
			dimerPos[1] = 0f;
			dimerPos[2] = 0f;
			protofilament.addObject(dimer, dimerPos);
		}
		
		for (int i=0; i<protoLength-1; i++)
		{
			protofilament.addConnection(2*i+2, 2*i+1, 4f*nano, 0);
			//protofilament.addConnection(0, protoLength*2-1, 8f*(protoLength-0.5f)*nano, 0);
			//protofilament.addConnection(0, protoLength*2-1, 8f*(protoLength-0.5f)*nano, 0);
		}
		
		//float[] protoPos = {0f, 0f, 0f};
		//objectList[0] = protofilament.createInstance(protoPos);
		
		// microtubule
		ObjectTemplate microtubule = new ObjectTemplate();
		int protos = 13;
		float[] protoPos = new float[3];
		float theta;
		for (int i=0; i<protos; i++)
		{
			theta = 27.7f * (float) i;
			protoPos[0] = i * 0.923f * nano;
			protoPos[1] = 10f * nano * (float) Math.cos((double) theta * 3.141 / 180.0);
			protoPos[2] = 10f * nano * (float) Math.sin((double) theta * 3.141 / 180.0);;
			microtubule.addObject(protofilament, protoPos);
		}
		
		// connect adjacent tubulins
		for (int i=0; i<protos-1; i++)
		{
			for (int j=0; j<protoLength*2; j++)
			{
				microtubule.
					addConnection(i*protoLength*2+j, (i+1)*protoLength*2+j, 4.88f*nano, 0);
				
				//if (j>0)
				//{
				//	microtubule.
				//		addConnection(i*protoLength*2+j, (i+1)*protoLength*2+j-1, 
				//					  6.18f*nano, 0);
					
				//}
			}
		}
		
		// join up the seam
		for (int i=3; i<(protoLength*2); i++)
		{
			microtubule.addConnection(i, (protos-1)*protoLength*2+i-3, 4.88f*nano, 0);
		}
			
		// Create rigid spokes to prevent buckling
		for (int i=0; i<protoLength*2; i++)
		{
			int current=5; 
			int old=0;
			
			for (int j=0; j<protos; j++)
			{
				microtubule.addConnection(old*protoLength*2+i, current*protoLength*2+i,
										  19.26f*nano, 0);
				if (i<protoLength*2-2)
				{
					microtubule.addConnection(old*protoLength*2+i,
					 	current*protoLength*2+i+2, 20.11f*nano, 0);
				}
									  
				old = current;
				current = (current+5)%13;
			}
		}
		*/
		/*
		// Create supports to prevent axial rotation
		float conDist = (float) Math.sqrt( 
							(((protoLength-1)*8f*nano+4.62f*nano) *
							 ((protoLength-1)*8f*nano+4.62f*nano)) +
							18.7f*nano*18.7f*nano);
		for (int k=0; k<20; k++)
		{
		for (int i=0; i<protos-5; i++)
		{
			microtubule.addConnection(i*protoLength*2,
							 ((i+5)*protoLength*2)+protoLength*2-1, 
							 conDist, 0);
		}
		}
		
		*/
	/*
		// Create a microtubule
		float[] microPos = {0f, 0f, 0f};
		objectList[0] = microtubule.createInstance(microPos);
	
		
		// Create the kinesin object
		
		// Motor domain
		ParticleTemplate head = new ParticleTemplate(particleTemplates);
		head.setColour(green);
		head.setRadiusVal(2.75f);
		head.setRadiusUnit(nano);
		head.setName("Head");
		
		// Neck
		ParticleTemplate neck = new ParticleTemplate(particleTemplates);
		neck.setColour(green);
		neck.setRadiusVal(1.25f);
		neck.setRadiusUnit(nano);
		neck.setName("Neck");
		
		// Motor region
		ObjectTemplate motor = new ObjectTemplate();
		float[] head1Pos = {0f,0f,0f};
		float[] head2Pos = {0f,8f*nano,0f};
		float[] neckPos = {0f,4f*nano,0f};
		motor.addParticle(head, head1Pos);
		motor.addParticle(head, head2Pos);
		motor.addParticle(neck, neckPos);
		
		float[] kinesinPos = {0f, 4f*nano, 0f};
		objectList[1] = motor.createInstance(kinesinPos);
		
		int numObjects = objectList[0].length + objectList[1].length;
		objects = new SimObject[numObjects];
		for (int i=0; i<objectList[0].length; i++)
		{
			objects[i] = objectList[0][i];
		}
		for (int i=0; i<objectList[1].length; i++)
		{
			objects[i + objectList[0].length] = objectList[1][i];
		}
	}
	*/
	
	public void sampleParticleGroup(int protoLength)
	{
		// colours
		final float[] red = {1f, 0.3f, 0.3f};
		final float[] blue = {0.3f, 0.3f, 1f};
		final float[] green = {0.2f, 0.8f, 0.2f};
		
		// radii
		final float alphaRadius = 2f;
		final float betaRadius = 2f;
		
		// protofilament length (in dimer)
		if (protoLength == -1)
		{
			protoLength = 10;
		}
		
		//boundary = new Boundary("tessellated", 500f * nano);
		
		// Connection lengths
		final float dimerCon = alphaRadius*nano + betaRadius*nano;
		final float protoCon = alphaRadius*nano + betaRadius*nano;
		
		// Hinge forces
		final float protoHinge = 2f* (float) Math.pow(10.0, -23.0);
		
		particleTemplates = new Vector();
		SimObject[][] objectList = new SimObject[2][];
		
		// alpha particle
		ParticleTemplate alpha = new ParticleTemplate(particleTemplates);
		alpha.setColour(red);
		alpha.setRadiusVal(alphaRadius);
		alpha.setRadiusUnit(nano);
		alpha.setName("Alpha tubulin");
		
		// beta particle
		ParticleTemplate beta = new ParticleTemplate(particleTemplates);
		beta.setColour(blue);
		beta.setRadiusVal(betaRadius);
		beta.setRadiusUnit(nano);
		beta.setName("Beta tubulin");
			
		// dimer
		ObjectTemplate dimer = new ObjectTemplate(objectTemplates);
		dimer.setName("Dimer");
		float[] alphaPos = {0f,0f,0f};
		float[] betaPos= {alphaRadius*nano+betaRadius*nano,0f, 0f};
		dimer.addParticle(alpha, alphaPos);
		dimer.addParticle(beta, betaPos);
			
		//dimer.addConnection(0, 1, dimerCon, 0);
		
		ObjectTemplate protofilament = new ObjectTemplate(objectTemplates);
		protofilament.setName("Protofilament");
		float[] dimerPos = new float[3];
		for (int i=0; i<protoLength; i++)
		{
			dimerPos[0] = (2f*alphaRadius*nano + 2f*betaRadius*nano)*i;
			dimerPos[0] -= 4f*nano*protoLength;
			dimerPos[1] = 0f;
			dimerPos[2] = 0f;
			protofilament.addObject(dimer, dimerPos);
		}
		
		for (int i=0; i<protoLength-1; i++)
		{
			//protofilament.addConnection(2*i+1, 2*i+2, 4f*nano, 0);
		}
			
		//protofilament.addConnection(1, 2, dimerCon, 0);
		
		// Create a microtubule
		//float[] microPos = {0f, 0f, 0f};
		//objectList[0] = protofilament.createInstance(microPos);
		//addObjectInstance(protofilament, microPos);
		//rootObjects.add(objectList[0][0]);
		
		/*
		// protofilament
		ObjectTemplate protofilament = new ObjectTemplate();
		float[] dimerPos = new float[3];
		for (int i=0; i<protoLength; i++)
		{
			dimerPos[0] = (2f*alphaRadius*nano + 2f*betaRadius*nano)*i;
			dimerPos[0] -= 4f*nano*protoLength;
			dimerPos[1] = 0f;
			dimerPos[2] = 0f;
			protofilament.addObject(dimer, dimerPos);
		}
		
		for (int i=0; i<protoLength-1; i++)
		{
			protofilament.addConnection(2*i+2, 2*i+1, 4f*nano, 0);
			//protofilament.addConnection(0, protoLength*2-1, 8f*(protoLength-0.5f)*nano, 0);
			//protofilament.addConnection(0, protoLength*2-1, 8f*(protoLength-0.5f)*nano, 0);
		}
		
		//float[] protoPos = {0f, 0f, 0f};
		//objectList[0] = protofilament.createInstance(protoPos);
		*/
		// microtubule
		ObjectTemplate microtubule = new ObjectTemplate(objectTemplates);
		microtubule.setName("Microtubule");
		int protos = 13;
		float[] protoPos = new float[3];
		float theta;
		for (int i=0; i<protos; i++)
		{
			theta = 27.7f * (float) i;
			protoPos[0] = i * 0.923f * nano;
			protoPos[1] = 10f * nano * (float) Math.cos((double) theta * 3.141 / 180.0);
			protoPos[2] = 10f * nano * (float) Math.sin((double) theta * 3.141 / 180.0);;
			microtubule.addObject(protofilament, protoPos);
		}
		
		float[] microPos = {0f, 0f, 0f}; 
		addObjectInstance(microtubule, microPos);
		
		// connect adjacent tubulins
		for (int i=0; i<protos-1; i++)
		{
			for (int j=0; j<protoLength*2; j++)
			{
				//microtubule.
					//addConnection(i*protoLength*2+j, (i+1)*protoLength*2+j, 4.88f*nano, 0);
				
				//if (j>0)
				//{
				//	microtubule.
				//		addConnection(i*protoLength*2+j, (i+1)*protoLength*2+j-1, 
				//					  6.18f*nano, 0);
					
				//}
			}
		}
		
		// join up the seam
		for (int i=3; i<(protoLength*2); i++)
		{
			//microtubule.addConnection(i, (protos-1)*protoLength*2+i-3, 4.88f*nano, 0);
		}
			
		// Create rigid spokes to prevent buckling
		/*
		for (int i=0; i<protoLength*2; i++)
		{
			int current=5; 
			int old=0;
			
			for (int j=0; j<protos; j++)
			{
				microtubule.addConnection(old*protoLength*2+i, current*protoLength*2+i,
										  19.26f*nano, 0);
				if (i<protoLength*2-2)
				{
					microtubule.addConnection(old*protoLength*2+i,
					 	current*protoLength*2+i+2, 20.11f*nano, 0);
				}
									  
				old = current;
				current = (current+5)%13;
			}
		}
		*/
		/*
		// Create supports to prevent axial rotation
		float conDist = (float) Math.sqrt( 
							(((protoLength-1)*8f*nano+4.62f*nano) *
							 ((protoLength-1)*8f*nano+4.62f*nano)) +
							18.7f*nano*18.7f*nano);
		for (int k=0; k<20; k++)
		{
		for (int i=0; i<protos-5; i++)
		{
			microtubule.addConnection(i*protoLength*2,
							 ((i+5)*protoLength*2)+protoLength*2-1, 
							 conDist, 0);
		}
		}
		
		*/
	
		/*
		// Create a microtubule
		float[] microPos = {0f, 0f, 0f};
		objectList[0] = microtubule.createInstance(microPos);
		*/
		
		// Create the kinesin object
		
		// Motor domain
		ParticleTemplate head = new ParticleTemplate(particleTemplates);
		head.setColour(green);
		head.setRadiusVal(2.75f);
		head.setRadiusUnit(nano);
		head.setName("Head");
		
		// Neck
		ParticleTemplate neck = new ParticleTemplate(particleTemplates);
		neck.setColour(green);
		neck.setRadiusVal(1.25f);
		neck.setRadiusUnit(nano);
		neck.setName("Neck");
		
		// vesicle
		ParticleTemplate vesicle = new ParticleTemplate(particleTemplates);
		vesicle.setColour(blue);
		vesicle.setRadiusVal(20f);
		vesicle.setRadiusUnit(nano);
		vesicle.setName("Vesicle");
		
		// Motor region
		ObjectTemplate motor = new ObjectTemplate(objectTemplates);
		motor.setName("Kinesin");
		float[] head1Pos = {0f,4f*nano,0f};
		float[] head2Pos = {8f*nano,4f*nano,0f};
		float[] neckPos = {4f*nano,4f*nano,0f};
		motor.addParticle(head, head1Pos);
		motor.addParticle(head, head2Pos);
		motor.addParticle(neck, neckPos);
		
		Vector alphaVect = new Vector();
		alphaVect.addElement(alpha); 
		ParticleGroupTemplate alphaGroup = new ParticleGroupTemplate(-1, alphaVect);
	
		motor.addParticleGroup(alphaGroup);
		
		for (int i=0; i<20; i++)
		{
			float[] bodyPos = new float[3];
			bodyPos[0] = 4f*nano;
			bodyPos[1] = 4f*nano + 2.5f*nano*i;
			bodyPos[2] = 0f;
			motor.addParticle(neck, bodyPos);
		}
		
		float[] vesiclePos = {4f*nano,72f*nano,0f};
		motor.addParticle(vesicle, vesiclePos);
		
		// state 0 //
		motor.addConnection(0,2,4f*nano, 0);
		motor.addConnection(1,2,4f*nano, 0);
		motor.addConnection(0,1,7f*nano, 0);
		
		motor.addConnection(2,4, 2.5f*nano, 0);
		for (int i=1; i<20; i++)
		{
			motor.addConnection(3+i, 4+i, 2.5f*nano, 0); 
		}
		
		motor.addConnection(4,24,70f*nano, 0);
		motor.addConnection(23,24,22f*nano, 0);
		
		motor.addStateChange(1, 0);
		int[] cTPartGroups = {0, 3};
		float[] vals = {5f*nano};
		ConditionTemplate cT = 
			new ConditionTemplate("dist", "lessthan", cTPartGroups, vals, boundary);
		motor.addCondition(cT, 0, 0);
		
		// state 1 //
		motor.addState();
		
		motor.addConnection(0,2,4f*nano, 1);
		motor.addConnection(1,2,4f*nano, 1);
		motor.addConnection(0,1,7f*nano, 1);
		motor.addConnection(0,3, 4.75f*nano, 1);
		
		motor.addConnection(2,4, 2.5f*nano, 1);
		for (int i=1; i<20; i++)
		{
			motor.addConnection(3+i, 4+i, 2.5f*nano, 1); 
		}
		
		motor.addConnection(4,24,70f*nano, 1);
		motor.addConnection(23,24,22f*nano, 1);
		
		motor.addStateChange(2, 1);
		int[] cTPartGroups2 = {1, 3};
		ConditionTemplate cT2 = 
			new ConditionTemplate("dist", "lessthan", cTPartGroups2, vals, boundary);
		motor.addCondition(cT2, 0, 1);
		
		// state 2 //
		motor.addState();
		
		motor.addConnection(0,2,4f*nano, 2);
		motor.addConnection(1,2,4f*nano, 2);
		motor.addConnection(0,1,7f*nano, 2);
		motor.addConnection(1,3, 4.75f*nano, 2);
		
		motor.addConnection(2,4, 2.5f*nano, 2);
		for (int i=1; i<20; i++)
		{
			motor.addConnection(3+i, 4+i, 2.5f*nano, 2); 
		}
		
		motor.addConnection(2,24,75f*nano, 2);
		motor.addConnection(23,24,22f*nano, 2);
		
		motor.addStateChange(1, 2);
		//ConditionTemplate cT = 
			//new ConditionTemplate("dist", "lessthan", cTPartGroups, vals, boundary);
		motor.addCondition(cT, 0, 2);
		
		
		// end //
		Vector collisionVect = new Vector();
		collisionVect.addElement(alpha);
		collisionVect.addElement(beta);
		ParticleGroupTemplate collisionGroup = new ParticleGroupTemplate(-1, collisionVect);
		
		motor.addParticleGroup(collisionGroup);
		/*
		motor.addCollision(0,25,1);
		motor.addCollision(0,25,2);
		motor.addCollision(1,25,1);
		motor.addCollision(1,25,2);
		*/
		
		float[] kinesinPos = {0f, 11f*nano, 0f};
		//objectList[1] = motor.createInstance(kinesinPos);
		addObjectInstance(motor, kinesinPos);
		//rootObjects.add(objectList[1][0]);
	}
	
	public void sampleTransport(int protoLength)
	{
		// colours
		final float[] red = {1f, 0.3f, 0.3f};
		final float[] blue = {0.3f, 0.3f, 1f};
		final float[] green = {0.2f, 0.8f, 0.2f};
		
		// radii
		final float alphaRadius = 2f;
		final float betaRadius = 2f;
		
		// protofilament length (in dimer)
		if (protoLength == -1)
		{
			protoLength = 10;
		}
		
		//boundary = new Boundary("tessellated", 500f * nano);
		
		// Connection lengths
		final float dimerCon = alphaRadius*nano + betaRadius*nano;
		final float protoCon = alphaRadius*nano + betaRadius*nano;
		
		// Hinge forces
		final float protoHinge = 2f* (float) Math.pow(10.0, -23.0);
		
		particleTemplates = new Vector();
		SimObject[][] objectList = new SimObject[2][];
		
		// alpha particle
		ParticleTemplate alpha = new ParticleTemplate(particleTemplates);
		alpha.setColour(red);
		alpha.setRadiusVal(alphaRadius);
		alpha.setRadiusUnit(nano);
		alpha.setName("Alpha tubulin");
		
		// beta particle
		ParticleTemplate beta = new ParticleTemplate(particleTemplates);
		beta.setColour(blue);
		beta.setRadiusVal(betaRadius);
		beta.setRadiusUnit(nano);
		beta.setName("Beta tubulin");
			
		// dimer
		ObjectTemplate dimer = new ObjectTemplate(objectTemplates);
		dimer.setName("Dimer");
		float[] alphaPos = {0f,0f,0f};
		float[] betaPos= {alphaRadius*nano+betaRadius*nano,0f, 0f};
		dimer.addParticle(alpha, alphaPos);
		dimer.addParticle(beta, betaPos);
			
		dimer.addConnection(0, 1, dimerCon, 0);
		
		ObjectTemplate protofilament = new ObjectTemplate(objectTemplates);
		protofilament.setName("Protofilament");
		float[] dimerPos = new float[3];
		for (int i=0; i<protoLength; i++)
		{
			dimerPos[0] = (2f*alphaRadius*nano + 2f*betaRadius*nano)*i;
			dimerPos[0] -= 4f*nano*protoLength;
			dimerPos[1] = 0f;
			dimerPos[2] = 0f;
			protofilament.addObject(dimer, dimerPos);
		}
		
		for (int i=0; i<protoLength-1; i++)
		{
			protofilament.addConnection(2*i+1, 2*i+2, 4f*nano, 0);
		}
		
		float[] protoPos = {0f, 0f, 0f}; 
		addObjectInstance(protofilament, protoPos);
	
	
		// Create the kinesin object
		
		// Motor domain
		ParticleTemplate head = new ParticleTemplate(particleTemplates);
		head.setColour(green);
		head.setRadiusVal(2.75f);
		head.setRadiusUnit(nano);
		head.setName("Head");
		
		// Neck
		ParticleTemplate neck = new ParticleTemplate(particleTemplates);
		neck.setColour(green);
		neck.setRadiusVal(1.25f);
		neck.setRadiusUnit(nano);
		neck.setName("Neck");
		
		// vesicle
		ParticleTemplate vesicle = new ParticleTemplate(particleTemplates);
		vesicle.setColour(blue);
		vesicle.setRadiusVal(20f);
		vesicle.setRadiusUnit(nano);
		vesicle.setName("Vesicle");
		
		// Motor region
		ObjectTemplate motor = new ObjectTemplate(objectTemplates);
		motor.setName("Kinesin");
		float[] head1Pos = {0f,4f*nano,0f};
		float[] head2Pos = {8f*nano,4f*nano,0f};
		float[] neckPos = {4f*nano,4f*nano,0f};
		motor.addParticle(head, head1Pos);
		motor.addParticle(head, head2Pos);
		motor.addParticle(neck, neckPos);
		
		Vector alphaVect = new Vector();
		alphaVect.addElement(alpha); 
		ParticleGroupTemplate alphaGroup = new ParticleGroupTemplate(-1, alphaVect);
	
		motor.addParticleGroup(alphaGroup);
		
		for (int i=0; i<20; i++)
		{
			float[] bodyPos = new float[3];
			bodyPos[0] = 4f*nano;
			bodyPos[1] = 4f*nano + 2.5f*nano*i;
			bodyPos[2] = 0f;
			motor.addParticle(neck, bodyPos);
		}
		
		float[] vesiclePos = {4f*nano,72f*nano,0f};
		motor.addParticle(vesicle, vesiclePos);
		
		// state 0 //
		motor.addConnection(0,2,4f*nano, 0);
		motor.addConnection(1,2,4f*nano, 0);
		motor.addConnection(0,1,7.5f*nano, 0);
		
		motor.addConnection(2,4, 2.5f*nano, 0);
		for (int i=1; i<20; i++)
		{
			motor.addConnection(3+i, 4+i, 2.5f*nano, 0); 
		}
		
		motor.addConnection(4,24,70f*nano, 0);
		motor.addConnection(23,24,22f*nano, 0);
		
		motor.addStateChange(1, 0);
		int[] cTPartGroups = {0, 3};
		float[] vals = {5f*nano};
		ConditionTemplate cT = 
			new ConditionTemplate("dist", "lessthan", cTPartGroups, vals, boundary);
		motor.addCondition(cT, 0, 0);
		
		// state 1 //
		motor.addState();
		
		motor.addConnection(0,2,4f*nano, 1);
		motor.addConnection(1,2,4f*nano, 1);
		motor.addConnection(0,1,7f*nano, 1);
		motor.addConnection(0,3, 4.75f*nano, 1);
		
		motor.addConnection(2,4, 2.5f*nano, 1);
		for (int i=1; i<20; i++)
		{
			motor.addConnection(3+i, 4+i, 2.5f*nano, 1); 
		}
		
		motor.addConnection(4,24,70f*nano, 1);
		motor.addConnection(23,24,22f*nano, 1);
		
		motor.addStateChange(2, 1);
		int[] cTPartGroups2 = {1, 3};
		ConditionTemplate cT2 = 
			new ConditionTemplate("dist", "lessthan", cTPartGroups2, vals, boundary);
		motor.addCondition(cT2, 0, 1);
		
		// state 2 //
		motor.addState();
		
		motor.addConnection(0,2,4f*nano, 2);
		motor.addConnection(1,2,4f*nano, 2);
		motor.addConnection(0,1,7f*nano, 2);
		motor.addConnection(1,3, 4.75f*nano, 2);
		
		motor.addConnection(2,4, 2.5f*nano, 2);
		for (int i=1; i<20; i++)
		{
			motor.addConnection(3+i, 4+i, 2.5f*nano, 2); 
		}
		
		motor.addConnection(4,24,70f*nano, 2);
		motor.addConnection(23,24,22f*nano, 2);
		
		motor.addStateChange(1, 2);
		//ConditionTemplate cT = 
			//new ConditionTemplate("dist", "lessthan", cTPartGroups, vals, boundary);
		motor.addCondition(cT, 0, 2);
		
		
		// end //
		Vector collisionVect = new Vector();
		collisionVect.addElement(alpha);
		collisionVect.addElement(beta);
		ParticleGroupTemplate collisionGroup = new ParticleGroupTemplate(-1, collisionVect);
		
		motor.addParticleGroup(collisionGroup);
		
		motor.addCollision(0,25,1);
		motor.addCollision(0,25,2);
		
		motor.addCollision(1,25,1);
		motor.addCollision(1,25,2);
		
		for (int i=4; i<8; i++)
		{
			motor.addCollision(i,25,1);
			motor.addCollision(i,25,2);
		}
		
		
		float[] kinesinPos = {0f, 0f*nano, 0f};
		//objectList[1] = motor.createInstance(kinesinPos);
		addObjectInstance(motor, kinesinPos);
		//rootObjects.add(objectList[1][0]);
	}
	
	public void sampleElectrodimers()
	{
		// colours
		final float[] red = {1f, 0.3f, 0.3f};
		final float[] blue = {0.3f, 0.3f, 1f};
		final float[] green = {0.3f, 0.8f, 0.3f};
		
		// radii
		final float alphaRadius = 2f;
		final float betaRadius = 2f;
		
		//boundary = new Boundary("tessellated", 500f * nano);
		
		// Connection lengths
		final float dimerCon = alphaRadius*nano + betaRadius*nano;
		
		particleTemplates = new Vector();
		
		// positive particle
		ParticleTemplate alpha = new ParticleTemplate(particleTemplates);
		alpha.setColour(red);
		alpha.setRadiusVal(alphaRadius);
		alpha.setRadiusUnit(nano);
		alpha.setChargeVal(16f);
		alpha.setChargeUnit((float) Math.pow(10.0, -19.0));
		alpha.setName("Positive");
		
		// negative particle
		ParticleTemplate beta = new ParticleTemplate(particleTemplates);
		beta.setColour(blue);
		beta.setRadiusVal(betaRadius);
		beta.setRadiusUnit(nano);
		beta.setChargeVal(-16f);
		beta.setChargeUnit((float) Math.pow(10.0, -19.0));
		beta.setName("Negative");
		
		// neutral particle
		ParticleTemplate neutral = new ParticleTemplate(particleTemplates);
		neutral.setColour(green);
		neutral.setRadiusVal(betaRadius);
		neutral.setRadiusUnit(nano);
		neutral.setName("Neutral");
			
		// dimer
		ObjectTemplate posObj = new ObjectTemplate(objectTemplates);
		posObj.setName("Positive object");
		float[] alphaPos = {0f,0f,0f};
		float[] betaPos= {alphaRadius*nano+betaRadius*nano,0f, 0f};
		posObj.addParticle(alpha, alphaPos);
		posObj.addParticle(neutral, betaPos);
		posObj.addConnection(0, 1, dimerCon, 0);
		
		// dimer
		ObjectTemplate negObj = new ObjectTemplate(objectTemplates);
		negObj.setName("Negative object");
		negObj.addParticle(beta, alphaPos);
		negObj.addParticle(neutral, betaPos);
		negObj.addConnection(0, 1, dimerCon, 0);
		
		Vector posVect = new Vector();
		posVect.addElement(alpha);
		ParticleGroupTemplate posGroup = new ParticleGroupTemplate(-1, posVect);
		
		negObj.addParticleGroup(posGroup);
		negObj.addElectrostatic(0,2,0);
		
		Vector negVect = new Vector();
		negVect.addElement(beta);
		ParticleGroupTemplate negGroup = new ParticleGroupTemplate(-1, negVect);
		
		posObj.addParticleGroup(negGroup);
		posObj.addElectrostatic(0,2,0);
	
		float[] dimerPos1 = {50f * nano, 0f, 0f}; 
		addObjectInstance(posObj, dimerPos1);
		float[] dimerPos2 = {-50f * nano, 0f, 0f}; 
		addObjectInstance(negObj, dimerPos2);
	}
}