import java.util.*;

import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import javax.swing.JTabbedPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.ImageIcon;
import javax.swing.*;
import javax.swing.tree.*;

import java.awt.*;
import java.awt.event.*;

public class SimulationGroup extends BranchGroup implements Runnable
{
	String[] args;

	Boundary boundary;
	Thread simulationThread;
	boolean simulationRunning;
	int frameDelay;
	float simulationTime;
	float realTime;
	//float simTime;
	//float simUnit;

	float frameTime;
	float frameUnit;

	float boundVal;
	float boundUnit;

	int iterations;

	boolean brownianOn, electrostaticOn;

        // Create a variable to hold the current algorithm
        final int DISSERTATION_ALGORITHM = 0;
        final int MONTE_CARLO = 1;
        final int DISSIPATIVE_PARTICLE_DYNAMICS = 2;
        int updateAlgorithm;

	JLabel simElapsedLab, frameLab;

	// Constants introduced for efficiency
	final float nano = (float) Math.pow(10.0, -9.0);

	///////////////////////////////
	Particle[] part;
	SimObject[] objects;
	float[][] pos;
	int numParticles;
	Medium medium;
	Random r;
	long seed;
	SimulationObj simulation;
	///////////////////////////////

	public SimulationGroup(Boundary b, Medium m, SimObject[] ob, Vector particleTemplates,
						   String[] a, int alg)
	{
		super();
		boundVal = 500f;
		boundUnit = (float) Math.pow(10.0, -9.0);
		boundary = b;
		medium = m;
		objects = ob;
		args = a;
                updateAlgorithm = alg;

		brownianOn = true;
		electrostaticOn = true;

		// Get the particles in the system - after all objects instantiated
		Vector particles = new Vector();
		for (int i=0; i<particleTemplates.size(); i++)
		{
			ParticleTemplate pt = (ParticleTemplate)(particleTemplates.elementAt(i));
			part = pt.getMembers();
			for (int j=0; j<part.length; j++)
			{
				particles.addElement(part[j]);
			}
		}
		part = new Particle[particles.size()];
		particles.copyInto(part);
		
		simulationThread = new Thread(this);
		frameDelay = 100;
		//simTime = 1f;
		//simUnit = (float) Math.pow(10.0, -6.0);
		
		frameTime = 1f;
		frameUnit = (float) Math.pow(10.0, -9.0);
		simulationTime = 0;
		realTime = 0;
		iterations = 500;
		
		simElapsedLab = new JLabel("");
		frameLab = new JLabel("");
 
		seed = 0;
		r = new Random();
		
		numParticles = part.length;
		
		simulation = new SimulationObj(objects, part, medium, boundary);
		
		simulation.showParticles(this);
		
		simulationRunning = false;
		simulationThread.start();
	}
	
	public void setIterations(int i)
	{
		iterations = i;
	}
	
	public boolean isBrowniannOn()
	{ 
		return brownianOn; 
	}

	public void setBrownianOn(boolean b)
	{
		brownianOn = b;
	}
	
	public boolean isElectrostaticOn()
	{ 
		return electrostaticOn; 
	}

	public void setElectrostaticOn(boolean e)
	{
		electrostaticOn = e;
	}
	
	public void setBoundVal(float val)
	{
		boundVal = val;
		boundary.setSimulationSize(boundVal * boundUnit);
	}
	
	public void setBoundUnit(float unit)
	{
		boundUnit = unit;
		boundary.setSimulationSize(boundVal * boundUnit);
	}
	
	public void setSeed(long s)
	{
		seed = s;
	}
	
	public void setFrameLabel(JLabel f)
	{
		frameLab = f;
	}
	
	public void setElapsedLabel(JLabel s)
	{
		simElapsedLab = s;
	}
	
	public void setTimeUnit(float tu)
	{
		frameUnit = tu;
	}
	
	public void setTimeValue(float tv)
	{
		frameTime = tv;
	}
	
	public void reset(int resetPos)
	{
		// restart the simulation
		simElapsedLab.setText("0");
		frameLab.setText("-");
		realTime = 0f;
		simulationTime = 0f;
		
		simulation.reset(resetPos);
		for (int i=0; i<numParticles; i++)
		{
			part[i].updateScene();
		}
	}
	
	public void enabled(boolean e)
	{
		simulationRunning = e;
	}
	
	public boolean isEnabled()
	{
		return simulationRunning;
	}
	
	public void run()
	{
		try
		{
			Date start;
			Date stop;
			//int timeout;
			//float simSecondsPerSecond = simTime * simUnit;
			
			while(true)
			{
				float frameTimeStep = frameTime * frameUnit;
				
				if (simulationRunning)
				{
					if (simulationTime == 0f)
					{
						r = new Random(seed);
					}
				
					start = new Date();
					//realTime += (frameDelay/1000f);
					//simulationTime += (frameDelay/1000f)*simSecondsPerSecond;
					simulationTime += frameTimeStep;
						
					//String rt = (realTime)+"000";
					//rt = rt.substring(0, rt.indexOf('.')) + "." +
					//	 rt.substring(rt.indexOf('.')+1, rt.indexOf('.')+3);
					//realElapsedLab.setText(""+rt);
		
					String st = (simulationTime/(float) frameUnit) + "000";
					st = st.substring(0, st.indexOf('.')) + "." +
						 st.substring(st.indexOf('.')+1, st.indexOf('.')+3);
					simElapsedLab.setText(""+st);
					

                                        // Call the appropriate update method
                                        if (updateAlgorithm == DISSERTATION_ALGORITHM)
                                        {
                                                this.updateWithDissertationAlgorithm();
                                        }
                                        else if (updateAlgorithm == MONTE_CARLO)
                                        {
                                                this.updateWithMonteCarlo();
                                        }
                                        else if (updateAlgorithm == DISSIPATIVE_PARTICLE_DYNAMICS)
                                        {
                                                this.updateWithDissipativeParticleDynamics();
                                        }

   					stop = new Date();
					float frameRate = 1000f/(stop.getTime() - start.getTime());
					frameRate = (float) (((int) (frameRate * 10f))/10f);
					frameLab.setText(""+frameRate);
					//timeout = frameDelay - (int) (stop.getTime() - start.getTime());
					//if (timeout<0) {timeout = 0;}
					simulationThread.sleep(5);
				}
				else
				{
					simulationThread.sleep(30);
				}
			}
		}
		catch (InterruptedException e) {}
	}

        public void updateWithMonteCarlo()
        {
                System.out.println("Monte Carlo Simulation.... Algorithm not ready");
        }

        public void updateWithDissipativeParticleDynamics()
        {
                System.out.println("Dissipative Particle Dynamics Simulation.... Algorithm not ready");
        }

	public void updateWithDissertationAlgorithm()
	{
		// Advance the simulation by one frame
		//float simSecondsPerSecond = simUnit * simTime;
		//float timestep = ((float)frameDelay/1000f)*simSecondsPerSecond;
		float timestep = frameUnit * frameTime;
		
		// Check for state changes
		simulation.checkStatesChanged();
		for (int i=0; i<numParticles; i++)
		{
			part[i].clearLists();
		}
		
		// Create the object structures
		simulation.apply();
		
		if (args.length>0)
		{ sampleApplications(); }
		
		Vector randomParticles = getRandomList();
		float[][] disp = new float[numParticles][3];
		int randParticle;
		
		for (int i=0; i<randomParticles.size(); i++)
		{
			randParticle = ((Integer) (randomParticles.elementAt(i))).intValue();
			
			disp[randParticle][0] = 0f;
			disp[randParticle][1] = 0f;
			disp[randParticle][2] = 0f;
			
			if (brownianOn)
			{
				part[randParticle].calculateBrownianDisplacement
										(timestep, disp[randParticle], r);	
			}
			
			if (electrostaticOn)
			{
				part[randParticle].calculateElectrostaticDisplacement
										(timestep, disp[randParticle]);
			}	
			part[randParticle].calculateHingedDisplacement(timestep, disp[randParticle]);
		}
		
		for (int i=0; i<randomParticles.size(); i++)
		{
			randParticle = ((Integer) (randomParticles.elementAt(i))).intValue();
			part[randParticle].collisionLimitedMove(disp[randParticle]);	
		}

		for (int j=0; j<iterations; j++)
		{
			try
			{
				if ((j%20) == 19)
				{
					simulationThread.sleep(2);
				}
			}
			catch (InterruptedException e)
			{}
		
			for (int i=0; i<randomParticles.size(); i++)
			{
				randParticle = ((Integer) (randomParticles.elementAt(i))).intValue();						
				part[randParticle].updatePosition(r);
			}
			if (randomParticles.size() > 0)
			{
				Integer rotate = (Integer) randomParticles.elementAt(0);
				randomParticles.removeElementAt(0);
				randomParticles.addElement(rotate);
			}
		}
	
		if (simulationRunning)
		{
			for (int i=0; i<randomParticles.size(); i++)
			{
				randParticle = ((Integer) (randomParticles.elementAt(i))).intValue();
				part[randParticle].updateScene();
			}
		}		
		
		/*
		float[] f = {10f * (float)Math.pow(10.0, -9.0)};
		Condition c = new Condition("dist","greaterthan", part, f, boundaryNode);
		Condition[] clist = {c};
		StateChange sc = new StateChange(clist, 2);
		System.out.println(""+sc.getState());
		*/
	}
	
	public Vector getRandomList()
	{
		// Create a randomly ordered list of particles
		Vector orderedParticles = new Vector(0);
		for (int i=0; i<numParticles; i++)
		{
			if (!part[i].isStatic())
			{
				orderedParticles.addElement(new Integer(i));
			}
		}
		Vector randomParticles = new Vector(0);
		Integer nextParticle;
		int randInt;
		
		while(orderedParticles.size() > 0)
		{
			randInt = r.nextInt(orderedParticles.size());
			nextParticle = (Integer) orderedParticles.elementAt(randInt);
			orderedParticles.removeElementAt(randInt);
			randomParticles.addElement(nextParticle);
		}
			
		return randomParticles;
	}
	
	public void setBoundary(Boundary b)
	{
		boundary = b;
	}
	
	public Boundary getBoundary()
	{
		return boundary; 
	}
	
	public void sampleApplications()
	{
		if (args[0].equals("-transport"))
		{
			for (int i=0; i<40; i++)
			{
				part[i].setStatic(true);
			}
		}
		
		if (args[0].equals("-particlegroup"))
		{
			for (int i=0; i<520; i++)
			{
				part[i].setStatic(true);
			}
		}
	}
}
