import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;
 
import CoordinateLayout.*;

public class SimulationPanel extends JPanel
{
	
	SimulationGenerator simGen;
	SimulationObjectList objects;
	SimulationObjectTemplateList templates;
	ObjectPositionTextField[] optf;
	ObjectPositionUnitMenu[] opum;
	CreateInstanceButton cib;
	
	public SimulationPanel(SimulationGenerator sgen)
	{
		super();
		
		simGen = sgen;
		
		setLayout(new BorderLayout());
		JPanel panel = new JPanel();
		panel.setPreferredSize(new Dimension(800, 200));
		panel.setLayout(new CoordinateLayout(new Dimension(25,8),4,4));
		
	 	cib = new CreateInstanceButton(simGen, this);
		panel.add(cib, new Coordinate(9,6,16,7));
		objects = new SimulationObjectList(simGen);
		templates = new SimulationObjectTemplateList(simGen, this);
		panel.add(new JLabel("Object templates:"), new Coordinate(1,0,8,1));
		panel.add(templates, new Coordinate(1,1,8,7));
		panel.add(new JLabel("Object instances:"), new Coordinate(17,0,24,1));
		panel.add(objects, new Coordinate(17,1,24,7));
		
		optf = new ObjectPositionTextField[3];
		opum = new ObjectPositionUnitMenu[3]; 
		optf[0] = new ObjectPositionTextField(); 
		optf[1] = new ObjectPositionTextField(); 
		optf[2] = new ObjectPositionTextField(); 
		opum[0] = new ObjectPositionUnitMenu();
		opum[1] = new ObjectPositionUnitMenu();
		opum[2] = new ObjectPositionUnitMenu();
		
		panel.add(new JLabel("X:"), new Coordinate(9,2,10,3));
		panel.add(optf[0], new Coordinate(10,2,12,3));
		panel.add(opum[0], new Coordinate(12,2,16,3));
		
		panel.add(new JLabel("Y:"), new Coordinate(9,3,10,4));
		panel.add(optf[1], new Coordinate(10,3,12,4));
		panel.add(opum[1], new Coordinate(12,3,16,4));
		
		panel.add(new JLabel("Z:"), new Coordinate(9,4,10,5));
		panel.add(optf[2], new Coordinate(10,4,12,5));
		panel.add(opum[2], new Coordinate(12,4,16,5));
		
		
		add(panel, BorderLayout.WEST);
	}
	
	public void refresh()
	{
		objects.refresh();
		templates.refresh();
	}
	
	public CreateInstanceButton getInstanceButton()
	{
		return cib;
	}
	
	public SimulationObjectList getObjectList()
	{
		return objects;
	}
	
	public int getSelectedTemplate()
	{
		return templates.getSelectedIndex();
	}
	
	public float[] getPos()
	{
		float[] pos = new float[3];
		
		pos[0] = optf[0].getVal() * opum[0].getUnit();
		pos[1] = optf[1].getVal() * opum[1].getUnit();
		pos[2] = optf[2].getVal() * opum[2].getUnit();
		
		return pos;
	}

}

class SimulationObjectTemplateList extends JPanel
{
	SimulationGenerator simGen;
	JList objectList;
	SimulationPanel panel;
	
	public SimulationObjectTemplateList(SimulationGenerator sgen, SimulationPanel p)
	{
		super();
		simGen = sgen;
		panel = p;
		
		objectList = new JList();
		objectList.setVisibleRowCount(7);
		objectList.setFixedCellWidth(150);
		objectList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		objectList.setEnabled(true);
		JScrollPane scrollPane = new JScrollPane(objectList);
		add(scrollPane, new Coordinate(0,0,8,7));
		refresh();
	}	
	
	public void refresh()
	{		
		Vector objects = simGen.getObjectTemplates();
		
		Vector names = new Vector();
		for (int i=0; i<objects.size(); i++)
		{
			names.add(((ObjectTemplate) (objects.elementAt(i))).getName());
		}
		int select = objectList.getSelectedIndex();
		objectList.setListData(names);
		objectList.setSelectedIndex(select);
	}

	public int getSelectedIndex()
	{
		return objectList.getSelectedIndex();
	}
}

class SimulationObjectList extends JPanel
{
	SimulationGenerator simGen;
	JList objectList;
	
	public SimulationObjectList(SimulationGenerator sgen)
	{
		super();
		simGen = sgen;
		
		objectList = new JList();
		objectList.setVisibleRowCount(7);
		objectList.setFixedCellWidth(150);
		objectList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		objectList.setEnabled(true);
		objectList.setSelectionForeground(new Color(0,0,0));
		objectList.setSelectionBackground(new Color(255,255,255));
		JScrollPane scrollPane = new JScrollPane(objectList);
		add(scrollPane);
		refresh();
	}
	
	public void refresh()
	{		
		Vector objects = simGen.getRootObjects();
		
		Vector names = new Vector();
		for (int i=0; i<objects.size(); i++)
		{
			names.add(((SimObject) (objects.elementAt(i))).getName());
		}
		int select = objectList.getSelectedIndex();
		objectList.setListData(names);
		objectList.setSelectedIndex(select);
	}
}

// addState button
class CreateInstanceButton extends JButton implements ActionListener
{
	SimulationPanel panel;
	SimulationGenerator simGen;
	
	public CreateInstanceButton(SimulationGenerator sg, SimulationPanel p)
	{
		super("Add instance >>>");
		panel = p;
		simGen = sg;
		addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		int selected = panel.getSelectedTemplate();
		if (selected != -1)
		{
			float[] pos = panel.getPos();
			SimObject[] subObjs;
			Vector templates = simGen.getObjectTemplates();
			simGen.addObjectInstance((ObjectTemplate)(templates.elementAt(selected)), pos);
			simGen.updateSimulation();
			panel.getObjectList().refresh();
		}
	}
}


class ObjectPositionTextField extends JTextField implements KeyListener
{
	String val;
	
	public ObjectPositionTextField()
	{
		super(6);
		enableField();
		setText("0");
		val = "0";
		
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}
	
	public void disableField()
	{
		setText("");
		val="";
		setEnabled(false);
	}
	
	public void enableField()
	{
		setEnabled(true);
	}
		
	public void keyPressed(KeyEvent ke)
	{}
	
	public void keyTyped(KeyEvent ke)
	{}
	
	public void keyReleased(KeyEvent ke)
	{
		String text = getText();
		try 
		{
			val = "" + (float) (Float.parseFloat(text));
		}
		catch (NumberFormatException e)
		{}
	}
	
	public void setVal(float v)
	{
		val = v + "";
		setText(val);
	}
	
	public float getVal()
	{
		try 
		{
			return (float) (Float.parseFloat(val));
		}
		catch (NumberFormatException e)
		{}
		
		return 0f;
	}
}

class ObjectPositionUnitMenu extends JComboBox implements ItemListener
{
	float radUnit;
	
	public ObjectPositionUnitMenu()
	{
		super();
		
		addItem("Metres");
		addItem("Millimetres");
		addItem("Micrometres");
		addItem("Nanometres");
		addItem("Picometres");
		addItem("Femtometres");
		
		setEditable(false);
		setUnit((float) Math.pow(10.0, -9.0));
	
		addItemListener(this);
	}
	
	public float getUnit()
	{
		return radUnit;
	}
	
	public void setUnit(float unit)
	{
		radUnit = unit;
		
		if (unit < 1.1f && unit > 0.9f) { setSelectedIndex(0); }
		if (unit < (float) Math.pow(10.0, -2.0) && unit > (float) Math.pow(10.0, -4.0)) 
			{ setSelectedIndex(1); }
		if (unit < (float) Math.pow(10.0, -5.0) && unit > (float) Math.pow(10.0, -7.0)) 
			{ setSelectedIndex(2); }
		if (unit < (float) Math.pow(10.0, -8.0) && unit > (float) Math.pow(10.0, -10.0)) 
			{ setSelectedIndex(3); }
		if (unit < (float) Math.pow(10.0, -11.0) && unit > (float) Math.pow(10.0, -13.0)) 
			{ setSelectedIndex(4); }
		if (unit < (float) Math.pow(10.0, -14.0) && unit > (float) Math.pow(10.0, -16.0)) 
			{ setSelectedIndex(5); }
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();
		
		if (s.equals("Metres")) { radUnit = 1f;  }
		if (s.equals("Millimetres")) 
			{ radUnit = (float) Math.pow(10.0, -3.0);  }
		if (s.equals("Micrometres")) 
			{ radUnit = (float) Math.pow(10.0, -6.0);  }
		if (s.equals("Nanometres")) 
			{ radUnit = (float) Math.pow(10.0, -9.0);  }
		if (s.equals("Picometres")) 
			{ radUnit = (float) Math.pow(10.0, -12.0);  }
		if (s.equals("Femtometres")) 
			{ radUnit = (float) Math.pow(10.0, -15.0);  }	
	}
	
	public void enableField()
	{ 
		removeItem("");
		setEnabled(true);
	}
	
	public void disableField()
	{
		addItem("");
		setSelectedItem("");
		setEnabled(false);
	}
}