import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

import CoordinateLayout.*;
import javax.swing.border.*;
import javax.swing.event.*;

public class Simulator extends JPanel
{
	static JTabbedPane tabbedPane;
	
	public Simulator()
	{}

	public static void main(String[] args)
	{
		try 
		{
    	    UIManager.setLookAndFeel(
        	    UIManager.getSystemLookAndFeelClassName());
    	} catch (Exception e) {}
		
		JFrame mainWindow = new JFrame("Protein Simulation Package - Dan Mossop, 2002.");
        mainWindow.setSize(800, 600);
		
		mainWindow.getContentPane().setLayout(new BorderLayout());
		
		mainWindow.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });
		 
		// Add simulation screen	
		SimulationCanvas simulationCanvas = new SimulationCanvas(args);
		mainWindow.getContentPane().add(simulationCanvas, BorderLayout.CENTER);
		
		SimulationGenerator simGen = simulationCanvas.getSimulationGenerator();
		tabbedPane = new JTabbedPane();
		
		// Create the side panel
		JPanel sidePanel = new JPanel();
		sidePanel.setLayout(new CoordinateLayout(new Dimension(1,13),8,8));
		sidePanel.setPreferredSize(new Dimension(200, 330));
		
		// Add the control panel
		ControlPanel controlPanel = new ControlPanel(simGen, tabbedPane);
		sidePanel.add(controlPanel, new Coordinate(0,0,1,7));
		
        mainWindow.setJMenuBar(new mainMenu(simGen, tabbedPane, controlPanel));
		
		// Add the random seed panel
		RandomPanel randomPanel = new RandomPanel(simGen);
		sidePanel.add(randomPanel, new Coordinate(0,7,1,10));
		
		JPanel sidePanelContainer = new JPanel();
		sidePanelContainer.setLayout(new BorderLayout());
		sidePanelContainer.add(sidePanel, BorderLayout.NORTH);
		mainWindow.getContentPane().add(sidePanelContainer, BorderLayout.EAST);
		
		// Add the boundary panel
		BoundaryPanel boundaryPanel = new BoundaryPanel(simGen);
		boundaryPanel.setPreferredSize(new Dimension(200,80));
		
		// Add the medium panel 
		MediumPanel mediumPanel = new MediumPanel(simGen);
		mediumPanel.setPreferredSize(new Dimension(200,150));
		
		// Add the forces panel 
		ForcesCheckBox forces = new ForcesCheckBox(simGen);
		forces.setPreferredSize(new Dimension(200,150));
		sidePanel.add(forces, new Coordinate(0,10,1,13));
		
		// Create a panel containing the medium and boundary
		JPanel medAndBound = new JPanel();
		medAndBound.setLayout(new CoordinateLayout(new Dimension(14,3),8,8));
		medAndBound.setPreferredSize(new Dimension(600, 0));
		medAndBound.add(mediumPanel, new Coordinate(2,0,7,3));
		medAndBound.add(boundaryPanel, new Coordinate(9,0,14,2));
		
		JPanel medAndBoundContainer = new JPanel();
		medAndBoundContainer.setLayout(new BorderLayout());
		medAndBoundContainer.add(medAndBound, BorderLayout.WEST);
		JPanel spacer = new JPanel();
		spacer.setPreferredSize(new Dimension(40,40));
		medAndBoundContainer.add(spacer, BorderLayout.NORTH);
		JPanel spacer2 = new JPanel();
		spacer2.setPreferredSize(new Dimension(40,40));
		medAndBoundContainer.add(spacer2, BorderLayout.SOUTH);
		
		// Create a tabbed pane
		final SimulationPanel simPanel = new SimulationPanel(simGen);
		tabbedPane.setPreferredSize(new Dimension(100, 220));
		tabbedPane.addTab("Medium and Boundary", medAndBoundContainer);
		final ParticleTemplatePanel partTemplatePanel = new ParticleTemplatePanel(simGen); 
		tabbedPane.addTab("Particle Templates", partTemplatePanel);
		final ObjectTemplatePanel objTemplatePanel = new ObjectTemplatePanel(simGen);
		tabbedPane.addTab("Object Templates", objTemplatePanel);
		final ParticleGroupPanel particleGroupPanel = new ParticleGroupPanel(simGen);
		tabbedPane.addTab("Particle Groups", particleGroupPanel);
		final ElectrostaticPanel electrostaticPanel = new ElectrostaticPanel(simGen);
		tabbedPane.addTab("Electrostatic", electrostaticPanel);
		tabbedPane.addTab("Simulation", simPanel);
		
		tabbedPane.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e)
			{
				simPanel.refresh();
				objTemplatePanel.refresh();
				partTemplatePanel.refresh();
				particleGroupPanel.refresh();
				electrostaticPanel.refresh();
			}
			});
		
		mainWindow.getContentPane().add(tabbedPane, BorderLayout.SOUTH);
		
        mainWindow.setVisible(true);
	}
}