import java.util.*;

public class StateTemplate
{
	Vector hingeTemps;
	Vector electroTemps;
	Vector connectTemps;
	Vector collisionTemps;
	Vector stateChangeTemps;

	public StateTemplate()
	{
		hingeTemps = new Vector();
		electroTemps = new Vector();
		connectTemps = new Vector();
		collisionTemps = new Vector();
		stateChangeTemps = new Vector();
	}
	 
	public StateTemplate copy(int a)
	{
		StateTemplate copy = new StateTemplate();
		
		Vector hingeCopy = new Vector();
		for (int i=0; i<hingeTemps.size(); i++)
		{
			HingeObjTemplate hinTemp = 
						(HingeObjTemplate) (hingeTemps.elementAt(i));
			hingeCopy.addElement(hinTemp.copy(a));
		}
		copy.setHingeTemplates(hingeCopy);
		
		Vector electroCopy = new Vector();
		for (int i=0; i<electroTemps.size(); i++)
		{
			ElectrostaticObjTemplate electroTemp = 
						(ElectrostaticObjTemplate) (electroTemps.elementAt(i));
			electroCopy.addElement(electroTemp.copy(a));
		}
		copy.setElectrostaticTemplates(electroCopy);
		
		Vector connectCopy = new Vector();
		for (int i=0; i<connectTemps.size(); i++)
		{
			ConnectionObjTemplate conTemp = 
						(ConnectionObjTemplate) (connectTemps.elementAt(i));
			connectCopy.addElement(conTemp.copy(a));
		}
		copy.setConnectionTemplates(connectCopy);
		
		Vector collisionCopy = new Vector();
		for (int i=0; i<collisionTemps.size(); i++)
		{
			CollisionObjTemplate collTemp = 
						(CollisionObjTemplate) (collisionTemps.elementAt(i));
			collisionCopy.addElement(collTemp.copy(a));
		}
		copy.setCollisionTemplates(collisionCopy);
		
		Vector stateChangeCopy = new Vector();
		for (int i=0; i<stateChangeTemps.size(); i++)
		{
			StateChangeTemplate scTemp = 
						(StateChangeTemplate) (stateChangeTemps.elementAt(i));
			stateChangeCopy.addElement(scTemp.copy(a));
		}
		copy.setStateChangeTemplates(stateChangeCopy);
		
		return copy;
	}
	
	public void setStateChangeTemplates(Vector scT)
	{
		stateChangeTemps = scT;
	}
	
	public void setHingeTemplates(Vector hT)
	{
		hingeTemps = hT;
	}
	
	public void setElectrostaticTemplates(Vector eT)
	{
		electroTemps = eT;
	}
	
	public void setConnectionTemplates(Vector cT)
	{
		connectTemps = cT;
	}
	
	public void setCollisionTemplates(Vector s)
	{
		collisionTemps = s;
	}
	
	public void addConnection(int a, int b, float d)
	{
		connectTemps.addElement(new ConnectionObjTemplate(a, b, d));
	}
	
	public void addHinge(int a, int b, int c, float d)
	{
		hingeTemps.addElement(new HingeObjTemplate(a, b, c, d));
	}
	
	public void addElectrostatic(int a, int b)
	{
		electroTemps.addElement(new ElectrostaticObjTemplate(a, b));
	}
	
	public void addCollision(int a, int b)
	{
		collisionTemps.addElement(new CollisionObjTemplate(a, b));
	}
	
	public void addStateChange(int newState)
	{
		stateChangeTemps.addElement(new StateChangeTemplate(newState));
	}
	
	public void addCondition(ConditionTemplate cT, int stateChange)
	{
		((StateChangeTemplate)(stateChangeTemps.elementAt(stateChange))).addCondition(cT);
	}

	public ObjectState createInstance(Vector particles)
	{
		ConnectionObj[] connectionObj = new ConnectionObj[connectTemps.size()];
		ConnectionObjTemplate tempConnect;
		for (int i=0; i<connectTemps.size(); i++)
		{
			tempConnect = (ConnectionObjTemplate) (connectTemps.elementAt(i));
			connectionObj[i] = tempConnect.createInstance(particles);
		}
		
		HingeObj[] hingeObj = new HingeObj[hingeTemps.size()];
		HingeObjTemplate tempHinge;
		for (int i=0; i<hingeTemps.size(); i++)
		{
			tempHinge = (HingeObjTemplate) (hingeTemps.elementAt(i));
			hingeObj[i] = tempHinge.createInstance(particles);
		}
		
		ElectrostaticObj[] electroObj = new ElectrostaticObj[electroTemps.size()];
		ElectrostaticObjTemplate tempElectro;
		for (int i=0; i<electroTemps.size(); i++)
		{
			tempElectro = (ElectrostaticObjTemplate) (electroTemps.elementAt(i));
			electroObj[i] = tempElectro.createInstance(particles);
		}
		
		CollisionObj[] collisionObj = new CollisionObj[collisionTemps.size()];
		CollisionObjTemplate tempCollision;
		for (int i=0; i<collisionTemps.size(); i++)
		{
			tempCollision = (CollisionObjTemplate) (collisionTemps.elementAt(i));
			collisionObj[i] = tempCollision.createInstance(particles);
		}
		
		StateChange[] stateChange = new StateChange[stateChangeTemps.size()];
		StateChangeTemplate tempStateChange;
		for (int i=0; i<stateChangeTemps.size(); i++)
		{
			tempStateChange = (StateChangeTemplate) (stateChangeTemps.elementAt(i));
			stateChange[i] = tempStateChange.createInstance(particles);
		}
		
		ObjectState stateInstance = new ObjectState(hingeObj, electroObj,
										connectionObj, collisionObj, stateChange);
										
		return stateInstance;
	}
}