import java.util.*;

public class StructureTemplate
{
	Vector stateTemps;
	int currentState;

	public StructureTemplate()
	{
		stateTemps = new Vector();
		stateTemps.addElement(new StateTemplate());
		currentState = 0;
	}
	
	public StructureTemplate copy(int a)
	{
		StructureTemplate copy = new StructureTemplate();
		copy.setCurrentState(currentState);
		Vector stateTempCopy = new Vector();
		for (int i=0; i<stateTemps.size(); i++)
		{
			StateTemplate sTCopy = ((StateTemplate) (stateTemps.elementAt(i))).copy(a);
			stateTempCopy.addElement(sTCopy);
		}
		
		copy.setStateTemplates(stateTempCopy);
		return copy;
	}
	 
	public void setStateTemplates(Vector s)
	{
		stateTemps = s;
	}
	
	public void setCurrentState(int s)
	{
		currentState = s;
	}
	
	public int addState()
	{
		stateTemps.addElement(new StateTemplate());
		return stateTemps.size();
	}
	
	public Vector getStates()
	{
		return stateTemps;
	}
	
	public void addConnection(int a, int b, float d, int state)
	{
		if (state<stateTemps.size())
		{
			((StateTemplate)(stateTemps.elementAt(state))).addConnection(a, b, d);
		}
		else
		{
			System.out.println("Attempt to add a connection to a non-existant state.");
		}
	}
	
	public void addCollision(int a, int b, int state)
	{
		if (state<stateTemps.size())
		{
			((StateTemplate)(stateTemps.elementAt(state))).addCollision(a, b);
		}
		else
		{
			System.out.println("Attempt to add a collision to a non-existant state.");
		}
	}
	
	public void addHinge(int a, int b, int c, float d, int state)
	{
		if (state<stateTemps.size())
		{
			((StateTemplate)(stateTemps.elementAt(state))).addHinge(a, b, c, d);
		}
		else
		{
			System.out.println("Attempt to add a hinge to a non-existant state.");
		}
	}
	
	public void addElectrostatic(int a, int b, int state)
	{
		if (state<stateTemps.size())
		{
			((StateTemplate)(stateTemps.elementAt(state))).addElectrostatic(a, b);
		}
		else
		{
			System.out.println("Attempt to add an electrostatic connection to a non-existant state.");
		}
	}
	
	public void addStateChange(int newState, int state)
	{
		if (state<stateTemps.size())
		{
			((StateTemplate)(stateTemps.elementAt(state))).addStateChange(newState);
		}
		else
		{
			System.out.println("Attempt to add a stateChange to a non-existant state.");
		}
	}
	
	public void addCondition(ConditionTemplate cT, int stateChange, int state)
	{
		if (state<stateTemps.size())
		{
			((StateTemplate)(stateTemps.elementAt(state))).addCondition(cT, stateChange);
		}
		else
		{
			System.out.println("Attempt to add a condition to a non-existant state.");
		}
	}
	
	public SimObject createInstance(Vector particles, ObjectTemplate ot)
	{
		ObjectState[] states = new ObjectState[stateTemps.size()];
		for (int i=0; i<stateTemps.size(); i++)
		{
			states[i] = ((StateTemplate) (stateTemps.elementAt(i))).createInstance(particles);
		}
		return (new SimObject(states, currentState, ot));
	}
}