import javax.swing.*;
import java.awt.event.*;

public class mainMenu extends JMenuBar
{
  	JMenu[] menu;
	SimulationGenerator simGen;
	JTabbedPane tabbedPane;
	ControlPanel controlPanel;
  
  public mainMenu(SimulationGenerator sg, JTabbedPane t, ControlPanel c)
  {
    super();

	simGen = sg;
	tabbedPane = t;
	controlPanel = c;
    JMenuItem menuItem;

    // Declare menus as heavyweight. This ensures that they are printed on top
    // the Java3d Canvas
    JPopupMenu.setDefaultLightWeightPopupEnabled(false);

    // declare an array of menus
    menu = new JMenu[3];

    ///////////// File menu ////////////////////
    menu[0] = new JMenu("File");
    menu[0].setMnemonic('F');

	// New simulation
    menuItem = new JMenuItem("New");
    menuItem.setMnemonic('n');
    menuItem.addActionListener(
      new ActionListener() 
      {
        public void actionPerformed(ActionEvent e)
        {
          simGen.newSimulation();
		  	
		  refresh();
        }
      }
    );
    menu[0].add(menuItem);

	// Exit
    menuItem = new JMenuItem("Exit");
    menuItem.setMnemonic('x');
    menuItem.addActionListener(
      new ActionListener() 
      {
        public void actionPerformed(ActionEvent e)
        {
          System.exit(0);
        }
      }
    );
    menu[0].add(menuItem);

    add(menu[0]);

    ///////////// Sample app menu ////////////////////
    menu[1] = new JMenu("Applications");
    menu[1].setMnemonic('A');

	// vesicle transportation
    menuItem = new JMenuItem("Vesicle Transportation - static microtubule; no collisions");
    menuItem.addActionListener(
      new ActionListener() 
      {
        public void actionPerformed(ActionEvent e)
        {
          simGen.newSimulation();
		  
		  String[] args = {"-particlegroup", "20"};
		  simGen.sampleApplications(args);
		  simGen.updateSimulation();
		  	
		  refresh();
        }
      }
    );
    menu[1].add(menuItem);

	// vesicle transportation - single proto
    menuItem = new JMenuItem("Vesicle Transportation - protofilament; collisions");
    menuItem.addActionListener(
      new ActionListener() 
      {
        public void actionPerformed(ActionEvent e)
        {
          simGen.newSimulation();
		  
		  String[] args = {"-transport", "20"};
		  simGen.sampleApplications(args);
		  simGen.updateSimulation();
		  	
		  refresh();
        }
      }
    );
    menu[1].add(menuItem);

	// microtubule
    menuItem = new JMenuItem("Microtubule");
    menuItem.addActionListener(
      new ActionListener() 
      {
        public void actionPerformed(ActionEvent e)
        {
          simGen.newSimulation();
		  
		  String[] args = {"-microtubule", "20"};
		  simGen.sampleApplications(args);
		  simGen.updateSimulation();
		  	
		  refresh();
        }
      }
    );
    menu[1].add(menuItem);
	
	add(menu[1]);

	// Electrodimers
    menuItem = new JMenuItem("Dimers - electrostatic");
    menuItem.addActionListener(
      new ActionListener() 
      {
        public void actionPerformed(ActionEvent e)
        {
          simGen.newSimulation();
		  
		  String[] args = {"-electrodimers"};
		  simGen.sampleApplications(args);
		  simGen.updateSimulation();
		  	
		  refresh();
        }
      }
    );
    menu[1].add(menuItem);
	
	add(menu[1]);

    //////////// Help menu /////////////////////
    menu[2] = new JMenu("Help");
    menu[2].setMnemonic('H'); 

    menuItem = new JMenuItem("About Simulator");
    menuItem.setMnemonic('A');
    menuItem.addActionListener(
      new ActionListener() 
      {
        public void actionPerformed(ActionEvent e)
        {
          JOptionPane.showMessageDialog(null,
		"A modelling package for protein interactions\n"
                +"Dan Mossop, 2002.",
          "About Simulator", JOptionPane.PLAIN_MESSAGE);
        }
      }
    );
    menu[2].add(menuItem);

    add(menu[2]);
  }
  
  public void refresh()
  {
	  ((ParticleTemplatePanel)(tabbedPane.getComponentAt(1))).resetIndex();
	  ((ParticleTemplatePanel)(tabbedPane.getComponentAt(1))).refresh();	
	  ((ObjectTemplatePanel)(tabbedPane.getComponentAt(2))).refresh();		
	  ((ParticleGroupPanel)(tabbedPane.getComponentAt(3))).refresh();		  
	  ((ElectrostaticPanel)(tabbedPane.getComponentAt(4))).refresh();  
	  ((SimulationPanel)(tabbedPane.getComponentAt(5))).refresh();
	  controlPanel.refresh();
  }
}

